@@ -31,6 +31,80 @@ along with pktools.  If not, see <http://www.gnu.org/licenses/>.
 #include "floatfann.h"
 #include "algorithms/myfann_cpp.h"
 
+/******************************************************************************/
+/*! \page pkann pkann
+ classify raster image using Artificial Neural Network
+## SYNOPSIS
+
+<code>
+  Usage: pkann -t training [-i input -o output] [-cv value]
+</code>
+
+<code>
+
+  
+  Options: [-tln layer]* [-c name -r value]* [-of GDALformat|-f OGRformat] [-co NAME=VALUE]* [-ct filename] [-label attribute] [-prior value]* [-nn number]* [-m filename [-msknodata value]*] [-nodata value]
+
+  Advanced options:
+       [-b band] [-sband band -eband band]* [-bal size]* [-min] [-bag value] [-bs value] [-comb rule] [-cb filename] [-prob filename] [-pim priorimage] [--offset value] [--scale value] [--connection 0|1] [-w weights]* [--learning rate] [--maxit number]
+</code>
+
+\section pkann_description Description
+
+The utility pkann implements an artificial neural network (ANN) to solve a supervised classification problem. The implementation is based on the open source C++ library <a href="http://leenissen.dk/fann/wp/">fann</a>). Both raster and vector files are supported as input. The output will contain the classification result, either in raster or vector format, corresponding to the format of the input. A training sample must be provided as an OGR vector dataset that contains the class labels and the features for each training point. The point locations are not considered in the training step. You can use the same training sample for classifying different images, provided the number of bands of the images are identical. Use the utility \ref pkextract "pkextract" to create a suitable training sample, based on a sample of points or polygons. For raster output maps you can attach a color table using the option -ct.
+
+\section pkann_options Options
+ - use either `-short` or `--long` options (both `--long=value` and `--long value` are supported)
+ - short option `-h` shows basic options only, long option `--help` shows all options
+|short|long|type|default|description|
+|-----|----|----|-------|-----------|
+ | i      | input                | std::string |       |input image | 
+ | t      | training             | std::string |       |training vector file. A single vector file contains all training features (must be set as: B0, B1, B2,...) for all classes (class numbers identified by label option). Use multiple training files for bootstrap aggregation (alternative to the bag and bsize options, where a random subset is taken from a single training file) | 
+ | tln    | tln                  | std::string |       |training layer name(s) | 
+ | label  | label                | std::string | label |identifier for class label in training vector file. | 
+ | bal    | balance              | unsigned int | 0     |balance the input data to this number of samples for each class | 
+ | min    | min                  | int  | 0     |if number of training pixels is less then min, do not take this class into account (0: consider all classes) | 
+ | b      | band                 | short |       |band index (starting from 0, either use band option or use start to end) | 
+ | sband  | startband            | unsigned short |      |Start band sequence number | 
+ | eband  | endband              | unsigned short |      |End band sequence number   | 
+ |        | offset               | double | 0     |offset value for each spectral band input features: refl[band]=(DN[band]-offset[band])/scale[band] | 
+ | scale  | scale                | double | 0     |scale value for each spectral band input features: refl=(DN[band]-offset[band])/scale[band] (use 0 if scale min and max in each band to -1.0 and 1.0) | 
+ | a      | aggreg               | unsigned short | 1     |how to combine aggregated classifiers, see also rc option (1: sum rule, 2: max rule). | 
+ | prior  | prior                | double | 0     |prior probabilities for each class (e.g., -p 0.3 -p 0.3 -p 0.2 ) | 
+ | pim    | priorimg             | std::string |       |prior probability image (multi-band img with band for each class | 
+ | cv     | cv                   | unsigned short | 0     |n-fold cross validation mode | 
+ | cmf    | cmf                  | std::string | ascii |Format for confusion matrix (ascii or latex) | 
+ | nn     | nneuron              | unsigned int | 5     |number of neurons in hidden layers in neural network (multiple hidden layers are set by defining multiple number of neurons: -n 15 -n 1, default is one hidden layer with 5 neurons) | 
+ |        | connection           | float | 1     |connection reate (default: 1.0 for a fully connected network) | 
+ | w      | weights              | float | 0     |weights for neural network. Apply to fully connected network only, starting from first input neuron to last output neuron, including the bias neurons (last neuron in each but last layer) | 
+ | l      | learning             | float | 0.7   |learning rate (default: 0.7) | 
+ |        | maxit                | unsigned int | 500   |number of maximum iterations (epoch) (default: 500) | 
+ | comb   | comb                 | unsigned short | 0     |how to combine bootstrap aggregation classifiers (0: sum rule, 1: product rule, 2: max rule). Also used to aggregate classes with rc option. Default is sum rule (0) | 
+ | bag    | bag                  | unsigned short | 1     |Number of bootstrap aggregations (default is no bagging: 1) | 
+ | bs     | bsize                | int  | 100   |Percentage of features used from available training features for each bootstrap aggregation (one size for all classes, or a different size for each class respectively | 
+ | cb     | classbag             | std::string |       |output for each individual bootstrap aggregation (default is blank) | 
+ | m      | mask                 | std::string |       |Only classify within specified mask (vector or raster). For raster mask, set nodata values with the option msknodata. | 
+ | msknodata | msknodata            | short | 0     |mask value(s) not to consider for classification (use negative values if only these values should be taken into account). Values will be taken over in classification image. Default is 0 | 
+ | nodata | nodata               | unsigned short | 0     |nodata value to put where image is masked as nodata | 
+ | o      | output               | std::string |       |output classification image | 
+ | ot     | otype                | std::string |       |Data type for output image ({Byte/Int16/UInt16/UInt32/Int32/Float32/Float64/CInt16/CInt32/CFloat32/CFloat64}). Empty string: inherit type from input image | 
+ | of     | oformat              | std::string |       |Output image format (see also gdal_translate). Empty string: inherit from input image | 
+ | ct     | ct                   | std::string |       |colour table in ASCII format having 5 columns: id R G B ALFA (0: transparent, 255: solid) | 
+ | co     | co                   | std::string |       |Creation option for output file. Multiple options can be specified. | 
+ |        | prob                 | std::string |       |probability image. Default is no probability image | 
+ | f      | f                    | std::string | SQLite |Output ogr format for active training sample | 
+ | na     | nactive              | unsigned int | 1     |number of active training points | 
+ | c      | class                | std::string |       |list of class names. | 
+ | r      | reclass              | short |       |list of class values (use same order as in class opt). | 
+
+Usage: pkann -t training [-i input -o output] [-cv value]
+
+
+Examples
+========
+Some examples how to use pkann can be found \ref examples_pkann "here"
+**/
+
 using namespace std;
 
 int main(int argc, char *argv[])
@@ -45,15 +119,16 @@ int main(int argc, char *argv[])
   Optionpk<unsigned int> balance_opt("bal", "balance", "balance the input data to this number of samples for each class", 0);
   Optionpk<bool> random_opt("random", "random", "in case of balance, randomize input data", true,2);
   Optionpk<int> minSize_opt("min", "min", "if number of training pixels is less then min, do not take this class into account (0: consider all classes)", 0);
-  Optionpk<short> band_opt("b", "band", "band index (starting from 0, either use band option or use start to end)");
-  Optionpk<double> bstart_opt("s", "start", "start band sequence number",0); 
-  Optionpk<double> bend_opt("e", "end", "end band sequence number (set to 0 to include bands)", 0); 
-  Optionpk<double> offset_opt("\0", "offset", "offset value for each spectral band input features: refl[band]=(DN[band]-offset[band])/scale[band]", 0.0);
-  Optionpk<double> scale_opt("\0", "scale", "scale value for each spectral band input features: refl=(DN[band]-offset[band])/scale[band] (use 0 if scale min and max in each band to -1.0 and 1.0)", 0.0);
+  Optionpk<unsigned short> band_opt("b", "band", "band index (starting from 0, either use band option or use start to end)");
+  Optionpk<unsigned short> bstart_opt("sband", "startband", "Start band sequence number"); 
+  Optionpk<unsigned short> bend_opt("eband", "endband", "End band sequence number"); 
+  Optionpk<double> offset_opt("offset", "offset", "offset value for each spectral band input features: refl[band]=(DN[band]-offset[band])/scale[band]", 0.0);
+  Optionpk<double> scale_opt("scale", "scale", "scale value for each spectral band input features: refl=(DN[band]-offset[band])/scale[band] (use 0 if scale min and max in each band to -1.0 and 1.0)", 0.0);
   Optionpk<unsigned short> aggreg_opt("a", "aggreg", "how to combine aggregated classifiers, see also rc option (1: sum rule, 2: max rule).",1);
   Optionpk<double> priors_opt("prior", "prior", "prior probabilities for each class (e.g., -p 0.3 -p 0.3 -p 0.2 )", 0.0); 
   Optionpk<string> priorimg_opt("pim", "priorimg", "prior probability image (multi-band img with band for each class","",2); 
   Optionpk<unsigned short> cv_opt("cv", "cv", "n-fold cross validation mode",0);
+  Optionpk<string> cmformat_opt("cmf","cmf","Format for confusion matrix (ascii or latex)","ascii");
   Optionpk<unsigned int> nneuron_opt("nn", "nneuron", "number of neurons in hidden layers in neural network (multiple hidden layers are set by defining multiple number of neurons: -n 15 -n 1, default is one hidden layer with 5 neurons)", 5); 
   Optionpk<float> connection_opt("\0", "connection", "connection reate (default: 1.0 for a fully connected network)", 1.0); 
   Optionpk<float> learning_opt("l", "learning", "learning rate (default: 0.7)", 0.7); 
@@ -63,7 +138,7 @@ int main(int argc, char *argv[])
   Optionpk<unsigned short> bag_opt("bag", "bag", "Number of bootstrap aggregations (default is no bagging: 1)", 1);
   Optionpk<int> bagSize_opt("bs", "bsize", "Percentage of features used from available training features for each bootstrap aggregation (one size for all classes, or a different size for each class respectively", 100);
   Optionpk<string> classBag_opt("cb", "classbag", "output for each individual bootstrap aggregation (default is blank)"); 
-  Optionpk<string> mask_opt("m", "mask", "Use the first band of the specified file as a validity mask. Nodata values can be set with the option msknodata.");
+  Optionpk<string> mask_opt("m", "mask", "Only classify within specified mask (vector or raster). For raster mask, set nodata values with the option msknodata.");
   Optionpk<short> msknodata_opt("msknodata", "msknodata", "mask value(s) not to consider for classification (use negative values if only these values should be taken into account). Values will be taken over in classification image. Default is 0", 0);
   Optionpk<unsigned short> nodata_opt("nodata", "nodata", "nodata value to put where image is masked as nodata", 0);
   Optionpk<string> output_opt("o", "output", "output classification image"); 
@@ -100,6 +175,8 @@ int main(int argc, char *argv[])
   maxit_opt.setHide(1);
   learning_opt.setHide(1);
 
+  verbose_opt.setHide(2);
+
   bool doProcess;//stop process when program was invoked with help option (-h --help)
   try{
     doProcess=input_opt.retrieveOption(argc,argv);
@@ -118,6 +195,7 @@ int main(int argc, char *argv[])
     priors_opt.retrieveOption(argc,argv);
     priorimg_opt.retrieveOption(argc,argv);
     cv_opt.retrieveOption(argc,argv);
+    cmformat_opt.retrieveOption(argc,argv);
     nneuron_opt.retrieveOption(argc,argv);
     connection_opt.retrieveOption(argc,argv);
     weights_opt.retrieveOption(argc,argv);
@@ -181,6 +259,30 @@ int main(int argc, char *argv[])
   if(verbose_opt[0]>=1)
     cout << "number of bootstrap aggregations: " << nbag << endl;
   
+  ImgReaderOgr extentReader;
+  OGRLayer  *readLayer;
+
+  double ulx=0;
+  double uly=0;
+  double lrx=0;
+  double lry=0;
+
+  bool maskIsVector=false;
+  if(mask_opt.size()){
+    try{
+      extentReader.open(mask_opt[0]);
+      maskIsVector=true;
+      readLayer = extentReader.getDataSource()->GetLayer(0);
+      if(!(extentReader.getExtent(ulx,uly,lrx,lry))){
+	cerr << "Error: could not get extent from " << mask_opt[0] << endl;
+	exit(1);
+      }
+    }
+    catch(string errorString){
+      maskIsVector=false;
+    }
+  }
+
   ImgWriterOgr activeWriter;
   if(active_opt.size()){
     ImgReaderOgr trainingReader(training_opt[0]);
@@ -215,6 +317,28 @@ int main(int argc, char *argv[])
       priors[iclass]/=normPrior;
   }
 
+  //convert start and end band options to vector of band indexes
+  try{
+    if(bstart_opt.size()){
+      if(bend_opt.size()!=bstart_opt.size()){
+	string errorstring="Error: options for start and end band indexes must be provided as pairs, missing end band";
+	throw(errorstring);
+      }
+      band_opt.clear();
+      for(int ipair=0;ipair<bstart_opt.size();++ipair){
+	if(bend_opt[ipair]<=bstart_opt[ipair]){
+	  string errorstring="Error: index for end band must be smaller then start band";
+	  throw(errorstring);
+	}
+	for(int iband=bstart_opt[ipair];iband<=bend_opt[ipair];++iband)
+	  band_opt.push_back(iband);
+      }
+    }
+  }
+  catch(string error){
+    cerr << error << std::endl;
+    exit(1);
+  }
   //sort bands
   if(band_opt.size())
     std::sort(band_opt.begin(),band_opt.end());
@@ -227,7 +351,7 @@ int main(int argc, char *argv[])
       classValueMap[classname_opt[iclass]]=classvalue_opt[iclass];
   }
   //----------------------------------- Training -------------------------------
-  ConfusionMatrix cm;
+  confusionmatrix::ConfusionMatrix cm;
   vector< vector<double> > offset(nbag);
   vector< vector<double> > scale(nbag);
   map<string,Vector2d<float> > trainingMap;
@@ -245,7 +369,7 @@ int main(int argc, char *argv[])
         if(band_opt.size())
           totalSamples=trainingReaderBag.readDataImageOgr(trainingMap,fields,band_opt,label_opt[0],tlayer_opt,verbose_opt[0]);
         else
-          totalSamples=trainingReaderBag.readDataImageOgr(trainingMap,fields,bstart_opt[0],bend_opt[0],label_opt[0],tlayer_opt,verbose_opt[0]);
+          totalSamples=trainingReaderBag.readDataImageOgr(trainingMap,fields,0,0,label_opt[0],tlayer_opt,verbose_opt[0]);
         if(trainingMap.size()<2){
           string errorstring="Error: could not read at least two classes from training file, did you provide class labels in training sample (see option label)?";
           throw(errorstring);
@@ -582,6 +706,8 @@ int main(int argc, char *argv[])
   }//for ibag
   if(cv_opt[0]>1){
     assert(cm.nReference());
+    cm.setFormat(cmformat_opt[0]);
+    cm.reportSE95(false);
     std::cout << cm << std::endl;
     cout << "class #samples userAcc prodAcc" << endl;
     double se95_ua=0;
@@ -629,22 +755,6 @@ int main(int argc, char *argv[])
       cerr << error << endl;
       exit(2);
     }
-    ImgReaderGdal maskReader;
-    if(mask_opt.size()){
-      try{
-        if(verbose_opt[0]>=1)
-          std::cout << "opening mask image file " << mask_opt[0] << std::endl;
-        maskReader.open(mask_opt[0]);
-      }
-      catch(string error){
-        cerr << error << endl;
-        exit(2);
-      }
-      catch(...){
-        cerr << "error catched" << endl;
-        exit(1);
-      }
-    }
     ImgReaderGdal priorReader;
     if(priorimg_opt.size()){
       try{
@@ -715,6 +825,47 @@ int main(int argc, char *argv[])
       cerr << error << endl;
     }
   
+    ImgWriterGdal maskWriter;
+
+    if(maskIsVector){
+      try{
+	maskWriter.open("/vsimem/mask.tif",ncol,nrow,1,GDT_Float32,imageType,option_opt);
+	maskWriter.GDALSetNoDataValue(nodata_opt[0]);
+        maskWriter.copyGeoTransform(testImage);
+        maskWriter.setProjection(testImage.getProjection());
+	vector<double> burnValues(1,1);//burn value is 1 (single band)
+	maskWriter.rasterizeOgr(extentReader,burnValues);
+	extentReader.close();
+	maskWriter.close();
+      }
+      catch(string error){
+        cerr << error << std::endl;
+        exit(2);
+      }
+      catch(...){
+        cerr << "error catched" << std::endl;
+        exit(1);
+      }
+      mask_opt.clear();
+      mask_opt.push_back("/vsimem/mask.tif");
+    }
+    ImgReaderGdal maskReader;
+    if(mask_opt.size()){
+      try{
+        if(verbose_opt[0]>=1)
+          std::cout << "opening mask image file " << mask_opt[0] << std::endl;
+        maskReader.open(mask_opt[0]);
+      }
+      catch(string error){
+        cerr << error << std::endl;
+        exit(2);
+      }
+      catch(...){
+        cerr << "error catched" << std::endl;
+        exit(1);
+      }
+    }
+
     for(int iline=0;iline<nrow;++iline){
       vector<float> buffer(ncol);
       vector<short> lineMask;
@@ -744,7 +895,7 @@ int main(int argc, char *argv[])
           }
         }
         else{
-          for(int iband=bstart_opt[0];iband<bstart_opt[0]+nband;++iband){
+          for(int iband=0;iband<nband;++iband){
             if(verbose_opt[0]==2)
               std::cout << "reading band " << iband << std::endl;
             assert(iband>=0);
@@ -788,13 +939,22 @@ int main(int argc, char *argv[])
       //process per pixel
       for(int icol=0;icol<ncol;++icol){
         assert(hpixel[icol].size()==nband);
+	bool doClassify=true;
         bool masked=false;
+	double geox=0;
+	double geoy=0;
+        if(maskIsVector){
+	  doClassify=false;
+	  testImage.image2geo(icol,iline,geox,geoy);
+	  //check enveloppe first
+	  if(uly>=geoy&&lry<=geoy&&ulx<=geox&&lrx>=geox){
+	    doClassify=true;
+	  }
+	}
         if(mask_opt.size()){
 	  //read mask
 	  double colMask=0;
 	  double rowMask=0;
-	  double geox=0;
-	  double geoy=0;
 
 	  testImage.image2geo(icol,iline,geox,geoy);
 	  maskReader.geo2image(geox,geoy,colMask,rowMask);
@@ -846,22 +1006,25 @@ int main(int argc, char *argv[])
 	    }
 	  }
 	  bool valid=false;
-	  for(int iband=0;iband<nband;++iband){
+	  for(int iband=0;iband<hpixel[icol].size();++iband){
 	    if(hpixel[icol][iband]){
 	      valid=true;
 	      break;
 	    }
 	  }
-	  if(!valid){
-	    if(classBag_opt.size())
-	      for(int ibag=0;ibag<nbag;++ibag)
-		classBag[ibag][icol]=nodata_opt[0];
-	    classOut[icol]=nodata_opt[0];
-	    continue;//next column
-	  }
+	  if(!valid)
+	    doClassify=false;
+
 	}
-        for(int iclass=0;iclass<nclass;++iclass)
+        for(short iclass=0;iclass<nclass;++iclass)
           probOut[iclass][icol]=0;
+	if(!doClassify){
+	  if(classBag_opt.size())
+	    for(int ibag=0;ibag<nbag;++ibag)
+	      classBag[ibag][icol]=nodata_opt[0];
+	  classOut[icol]=nodata_opt[0];
+	  continue;//next column
+	}
 	if(verbose_opt[0]>1)
 	  std::cout << "begin classification " << std::endl;
         //----------------------------------- classification -------------------
@@ -1161,21 +1324,21 @@ int main(int argc, char *argv[])
     }
     if(cm.nReference()){
       std::cout << cm << std::endl;
-      cout << "class #samples userAcc prodAcc" << endl;
-      double se95_ua=0;
-      double se95_pa=0;
-      double se95_oa=0;
-      double dua=0;
-      double dpa=0;
-      double doa=0;
-      for(short iclass=0;iclass<cm.nClasses();++iclass){
-	dua=cm.ua_pct(cm.getClass(iclass),&se95_ua);
-	dpa=cm.pa_pct(cm.getClass(iclass),&se95_pa);
-	cout << cm.getClass(iclass) << " " << cm.nReference(cm.getClass(iclass)) << " " << dua << " (" << se95_ua << ")" << " " << dpa << " (" << se95_pa << ")" << endl;
-      }
-      std::cout << "Kappa: " << cm.kappa() << std::endl;
-      doa=cm.oa_pct(&se95_oa);
-      std::cout << "Overall Accuracy: " << doa << " (" << se95_oa << ")"  << std::endl;
+      // cout << "class #samples userAcc prodAcc" << endl;
+      // double se95_ua=0;
+      // double se95_pa=0;
+      // double se95_oa=0;
+      // double dua=0;
+      // double dpa=0;
+      // double doa=0;
+      // for(short iclass=0;iclass<cm.nClasses();++iclass){
+      // 	dua=cm.ua_pct(cm.getClass(iclass),&se95_ua);
+      // 	dpa=cm.pa_pct(cm.getClass(iclass),&se95_pa);
+      // 	cout << cm.getClass(iclass) << " " << cm.nReference(cm.getClass(iclass)) << " " << dua << " (" << se95_ua << ")" << " " << dpa << " (" << se95_pa << ")" << endl;
+      // }
+      // std::cout << "Kappa: " << cm.kappa() << std::endl;
+      // doa=cm.oa_pct(&se95_oa);
+      // std::cout << "Overall Accuracy: " << doa << " (" << se95_oa << ")"  << std::endl;
     }
   }
   try{
