@@ -28,6 +28,90 @@ along with pktools.  If not, see <http://www.gnu.org/licenses/>.
 #include "base/Optionpk.h"
 #include "algorithms/StatFactory.h"
 
+/******************************************************************************/
+/*! \page pkcomposite pkcomposite
+
+ program to mosaic and composite geo-referenced images
+## SYNOPSIS
+
+<code>
+  Usage: pkcomposite -i input [-i input]* -o output
+</code>
+
+<code>
+
+  Options: [-b band]* [-dx xres] [-dy yres] [-e vector] [-ulx ULX -uly ULY -lrx LRX -lry LRY] [-cr rule] [-cb band] [-srcnodata value] [-bndnodata band] [-min value] [-max value] [-dstnodata value] [-r resampling_method] [-ot {Byte / Int16 / UInt16 / UInt32 / Int32 / Float32 / Float64 / CInt16 / CInt32 / CFloat32 / CFloat64}] [-of format] [-co NAME=VALUE]* [-a_srs epsg:number]
+
+  Advanced options:
+       [-file] [-w weight]* [-c class]* [-ct colortable] [-d description]
+</code>
+
+\section pkcomposite_description Description
+
+The utility pkcomposite can be used to \em mosaic and \em composite multiple (georeferenced) raster datasets. A mosaic can merge images with different geographical extents into a single larger image. Compositing resolves the overlapping pixels according to some rule (e.g, the median of all overlapping pixels). This utility is complementary to GDAL, which currently does not support a composite step. Input datasets can have different bounding boxes and spatial resolutionsresolution.
+
+\anchor pkcomposite_rules 
+composite rule | composite output
+------------- | -------------
+overwrite | Overwrite overlapping pixels: the latter input image on the command line overrules the previous image
+maxndvi | Create a maximum NDVI (normalized difference vegetation index) composite from multi-band input images. Use option -cb  to set the indexes of the red and near infrared bands respectively (e.g., -cb 0 -cb 1)
+maxband | Select the pixel with a maximum value in the band specified by option -cb
+minband | Select the pixel with a minimum value in the band specified by option -cb
+mean | Calculate the mean (average) of overlapping pixels
+stdev | Calculate the standard deviation of overlapping pixels
+median | Calculate the median of overlapping pixels
+mode | Select the mode of overlapping pixels (maximum voting): use for Byte images only
+sum | Calculate the arithmetic sum of overlapping pixels
+maxallbands | For each individual band, assign the maximum value found in all overlapping pixels. Unlike maxband, output band values cannot be attributed to a single (date) pixel in the input time series
+minallbands | For each individual band, assign the minimum value found in all overlapping pixels. Unlike minband, output band values cannot be attributed to a single (date) pixel in the input time series
+
+\section pkcomposite_options Options
+ - use either `-short` or `--long` options (both `--long=value` and `--long value` are supported)
+ - short option `-h` shows basic options only, long option `--help` shows all options
+
+|short|long|type|default|description|
+|-----|----|----|-------|-----------|
+ | i      | input                | std::string |       |Input image file(s). If input contains multiple images, a multi-band output is created | 
+ | o      | output               | std::string |       |Output image file | 
+ | b      | band                 | int  |       |band index(es) to crop (leave empty if all bands must be retained) | 
+ | dx     | dx                   | double |       |Output resolution in x (in meter) (empty: keep original resolution) | 
+ | dy     | dy                   | double |       |Output resolution in y (in meter) (empty: keep original resolution) | 
+ | e      | extent               | std::string |       |get boundary from extent from polygons in vector file | 
+ | cut      | crop_to_cutline    | bool | false |Crop the extent of the target dataset to the extent of the cutline | 
+ | m      | mask                 | std::string |       |Use the first band of the specified file as a validity mask (0 is nodata) | 
+ | msknodata | msknodata            | float | 0     |Mask value not to consider for composite
+ | mskband | mskband              | short | 0     |Mask band to read (0 indexed). Provide band for each mask. | 
+ | ulx    | ulx                  | double | 0     |Upper left x value bounding box | 
+ | uly    | uly                  | double | 0     |Upper left y value bounding box | 
+ | lrx    | lrx                  | double | 0     |Lower right x value bounding box | 
+ | lry    | lry                  | double | 0     |Lower right y value bounding box | 
+ | cr     | crule                | std::string | overwrite |Composite rule (overwrite, maxndvi, maxband, minband, mean, mode (only for byte images), median, sum, maxallbands, minallbands, stdev | 
+ | cb     | cband                | int  | 0     |band index used for the composite rule (e.g., for ndvi, use --cband=0 --cband=1 with 0 and 1 indices for red and nir band respectively | 
+ | srcnodata | srcnodata            | double |       |invalid value(s) for input raster dataset | 
+ | bndnodata | bndnodata            | int  | 0     |Band(s) in input image to check if pixel is valid (used for srcnodata, min and max options) | 
+ | min    | min                  | double |       |flag values smaller or equal to this value as invalid. | 
+ | max    | max                  | double |       |flag values larger or equal to this value as invalid. | 
+ | dstnodata | dstnodata            | double | 0     |nodata value to put in output raster dataset if not valid or out of bounds. | 
+ | r      | resampling-method    | std::string | near  |Resampling method (near: nearest neighbor, bilinear: bi-linear interpolation). | 
+ | ot     | otype                | std::string |       |Data type for output image ({Byte/Int16/UInt16/UInt32/Int32/Float32/Float64/CInt16/CInt32/CFloat32/CFloat64}). Empty string: inherit type from input image | 
+ | of     | oformat              | std::string |       |Output image format (see also gdal_translate). Empty string: inherit from input image | 
+ | co     | co                   | std::string |       |Creation option for output file. Multiple options can be specified. | 
+ | a_srs  | a_srs                | std::string |       |Override the spatial reference for the output file (leave blank to copy from input file, use epsg:3035 to use European projection and force to European grid | 
+ | file   | file                 | short | 0     |write number of observations (1) or sequence nr of selected file (2) for each pixels as additional layer in composite | 
+ | w      | weight               | short | 1     |Weights (type: short) for the composite, use one weight for each input file in same order as input files are provided). Use value 1 for equal weights. | 
+ | c      | class                | short | 0     |classes for multi-band output image: each band represents the number of observations for one specific class. Use value 0 for no multi-band output image. | 
+ | ct     | ct                   | std::string |       |color table file with 5 columns: id R G B ALFA (0: transparent, 255: solid) | 
+ | d      | description          | std::string |       |Set image description | 
+
+Examples
+========
+Some examples how to use pkcomposite can be found \ref examples_pkcomposite "here"
+
+FAQ
+===
+Frequently asked questions on pkcomposite can be found \ref faq_pkcomposite "here
+**/
+
 namespace crule{
   enum CRULE_TYPE {overwrite=0, maxndvi=1, maxband=2, minband=3, validband=4, mean=5, mode=6, median=7,sum=8,minallbands=9,maxallbands=10,stdev=11};
 }
@@ -42,6 +126,10 @@ int main(int argc, char *argv[])
   Optionpk<double>  dx_opt("dx", "dx", "Output resolution in x (in meter) (empty: keep original resolution)");
   Optionpk<double>  dy_opt("dy", "dy", "Output resolution in y (in meter) (empty: keep original resolution)");
   Optionpk<string>  extent_opt("e", "extent", "get boundary from extent from polygons in vector file");
+  Optionpk<bool> cut_opt("cut", "crop_to_cutline", "Crop the extent of the target dataset to the extent of the cutline.",false);
+  Optionpk<string> mask_opt("m", "mask", "Use the first band of the specified file as a validity mask (0 is nodata).");
+  Optionpk<float> msknodata_opt("msknodata", "msknodata", "Mask value not to consider for composite.", 0);
+  Optionpk<short> mskband_opt("mskband", "mskband", "Mask band to read (0 indexed). Provide band for each mask.", 0);
   Optionpk<double>  ulx_opt("ulx", "ulx", "Upper left x value bounding box", 0.0);
   Optionpk<double>  uly_opt("uly", "uly", "Upper left y value bounding box", 0.0);
   Optionpk<double>  lrx_opt("lrx", "lrx", "Lower right x value bounding box", 0.0);
@@ -65,12 +153,17 @@ int main(int argc, char *argv[])
   Optionpk<string>  description_opt("d", "description", "Set image description");
   Optionpk<short>  verbose_opt("v", "verbose", "verbose", 0,2);
 
+  extent_opt.setHide(1);
+  cut_opt.setHide(1);
+  mask_opt.setHide(1);
+  msknodata_opt.setHide(1);
+  mskband_opt.setHide(1);
+  option_opt.setHide(1);
   file_opt.setHide(1);
   weight_opt.setHide(1);
   class_opt.setHide(1);
   colorTable_opt.setHide(1);
   description_opt.setHide(1);
-  verbose_opt.setHide(1);
 
   bool doProcess;//stop process when program was invoked with help option (-h --help)
   try{
@@ -80,6 +173,10 @@ int main(int argc, char *argv[])
     dx_opt.retrieveOption(argc,argv);
     dy_opt.retrieveOption(argc,argv);
     extent_opt.retrieveOption(argc,argv);
+    cut_opt.retrieveOption(argc,argv);
+    mask_opt.retrieveOption(argc,argv);
+    msknodata_opt.retrieveOption(argc,argv);
+    mskband_opt.retrieveOption(argc,argv);
     ulx_opt.retrieveOption(argc,argv);
     uly_opt.retrieveOption(argc,argv);
     lrx_opt.retrieveOption(argc,argv);
@@ -166,6 +263,10 @@ int main(int argc, char *argv[])
     exit(1);
   }
   
+  if(input_opt.empty()){
+    std::cerr << "No input file provided (use option -i). Use --help for help information" << std::endl;
+    exit(0);
+  }
   int nband=0;
   int nwriteBand=0;
   int writeBand=0;
@@ -202,15 +303,41 @@ int main(int argc, char *argv[])
   //get bounding box from extentReader if defined
   ImgReaderOgr extentReader;
   if(extent_opt.size()){
-    extentReader.open(extent_opt[0]);
-    if(!(extentReader.getExtent(ulx_opt[0],uly_opt[0],lrx_opt[0],lry_opt[0]))){
-       cerr << "Error: could not get extent from " << extent_opt[0] << endl;
-       exit(1);
+    double e_ulx;
+    double e_uly;
+    double e_lrx;
+    double e_lry;
+    for(int iextent=0;iextent<extent_opt.size();++iextent){
+      extentReader.open(extent_opt[iextent]);
+      if(!(extentReader.getExtent(e_ulx,e_uly,e_lrx,e_lry))){
+        cerr << "Error: could not get extent from " << extent_opt[0] << endl;
+        exit(1);
       }
-    else if(verbose_opt[0])
-      cout << "--ulx=" << ulx_opt[0] << " --uly=" << uly_opt[0] << " --lrx=" << lrx_opt[0] << " --lry=" << lry_opt[0] << endl;
+      if(!iextent){
+	ulx_opt[0]=e_ulx;
+	uly_opt[0]=e_uly;
+	lrx_opt[0]=e_lrx;
+	lry_opt[0]=e_lry;
+      }
+      else{
+	if(e_ulx<ulx_opt[0])
+	  ulx_opt[0]=e_ulx;
+	if(e_uly>uly_opt[0])
+	  uly_opt[0]=e_uly;
+	if(e_lrx>lrx_opt[0])
+	  lrx_opt[0]=e_lrx;
+	if(e_lry<lry_opt[0])
+	  lry_opt[0]=e_lry;
+      }
+      extentReader.close();
+    }
+    if(cut_opt.size())
+      extentReader.open(extent_opt[0]);
   }
 
+  if(verbose_opt[0])
+    cout << "--ulx=" << ulx_opt[0] << " --uly=" << uly_opt[0] << " --lrx=" << lrx_opt[0] << " --lry=" << lry_opt[0] << endl;
+
   ImgReaderGdal imgReader;
   string theProjection="";
   GDALColorTable* theColorTable=NULL;
@@ -454,6 +581,65 @@ int main(int argc, char *argv[])
     else if(theColorTable)
       imgWriter.setColorTable(theColorTable);
   }
+
+  ImgWriterGdal maskWriter;
+  if(extent_opt.size()&&cut_opt[0]){
+    try{
+      maskWriter.open("/vsimem/mask.tif",ncol,nrow,1,GDT_Float32,"GTiff",option_opt);
+      double gt[6];
+      gt[0]=minULX;
+      gt[1]=dx;
+      gt[2]=0;
+      gt[3]=maxULY;
+      gt[4]=0;
+      gt[5]=-dy;
+      maskWriter.setGeoTransform(gt);
+      if(projection_opt.size())
+	maskWriter.setProjectionProj4(projection_opt[0]);
+      else if(theProjection!=""){
+	if(verbose_opt[0])
+	  cout << "projection: " << theProjection << endl;
+	maskWriter.setProjection(theProjection);
+      }
+	
+      //todo: handle multiple extent options
+      vector<double> burnValues(1,1);//burn value is 1 (single band)
+      maskWriter.rasterizeOgr(extentReader,burnValues);
+      maskWriter.close();
+    }
+    catch(string error){
+      cerr << error << std::endl;
+      exit(2);
+    }
+    catch(...){
+      cerr << "error catched" << std::endl;
+      exit(1);
+    }
+    //todo: support multiple masks
+    mask_opt.clear();
+    mask_opt.push_back("/vsimem/mask.tif");
+  }
+  ImgReaderGdal maskReader;
+  if(mask_opt.size()){
+    try{
+      if(verbose_opt[0]>=1)
+	std::cout << "opening mask image file " << mask_opt[0] << std::endl;
+      maskReader.open(mask_opt[0]);
+      if(mskband_opt[0]>=maskReader.nrOfBand()){
+	string errorString="Error: illegal mask band";
+	throw(errorString);
+      }
+    }
+    catch(string error){
+      cerr << error << std::endl;
+      exit(2);
+    }
+    catch(...){
+      cerr << "error catched" << std::endl;
+      exit(1);
+    }
+  }
+
   //create composite image
   if(verbose_opt[0])
      cout << "creating composite image" << endl;
@@ -480,9 +666,16 @@ int main(int argc, char *argv[])
   double progress=0;
   pfnProgress(progress,pszMessage,pProgressArg);
   for(int irow=0;irow<imgWriter.nrOfRow();++irow){
+    vector<float> lineMask;
     Vector2d< vector<double> > storeBuffer;
     vector<bool> writeValid(ncol);
 
+    //convert irow to geo
+    double x=0;
+    double y=0;
+    imgWriter.image2geo(0,irow,x,y);
+
+
     if(cruleMap[crule_opt[0]]==crule::mean ||
        cruleMap[crule_opt[0]]==crule::median ||
        cruleMap[crule_opt[0]]==crule::sum ||
@@ -502,6 +695,9 @@ int main(int argc, char *argv[])
           writeBuffer[iband][icol]=dstnodata_opt[0];
       }
     }
+
+    double oldRowMask=-1;//keep track of row mask to optimize number of line readings
+
     for(int ifile=0;ifile<input_opt.size();++ifile){
       try{
         imgReader.open(input_opt[ifile]);
@@ -535,10 +731,6 @@ int main(int argc, char *argv[])
         endCol=imgReader.nrOfCol()-1;
       int readncol=endCol-startCol+1;
 
-      //convert irow to geo
-      double x=0;
-      double y=0;
-      imgWriter.image2geo(0,irow,x,y);
       //lookup corresponding row for irow in this file
       imgReader.geo2image(x,y,readCol,readRow);
       if(readRow<0||readRow>=imgReader.nrOfRow()){
@@ -559,7 +751,42 @@ int main(int argc, char *argv[])
       }
         
       for(int ib=0;ib<ncol;++ib){
-        assert(imgWriter.image2geo(ib,irow,x,y));
+        imgWriter.image2geo(ib,irow,x,y);
+	//check mask first
+	bool valid=true;
+	if(mask_opt.size()){
+	  //read mask
+	  double colMask=0;
+	  double rowMask=0;
+
+	  maskReader.geo2image(x,y,colMask,rowMask);
+	  colMask=static_cast<int>(colMask);
+	  rowMask=static_cast<int>(rowMask);
+	  if(rowMask>=0&&rowMask<maskReader.nrOfRow()&&colMask>=0&&colMask<maskReader.nrOfCol()){
+	    if(static_cast<int>(rowMask)!=static_cast<int>(oldRowMask)){
+
+	      assert(rowMask>=0&&rowMask<maskReader.nrOfRow());
+	      try{
+		maskReader.readData(lineMask,GDT_Float32,static_cast<int>(rowMask),mskband_opt[0]);
+	      }
+	      catch(string errorstring){
+		cerr << errorstring << endl;
+		exit(1);
+	      }
+	      catch(...){
+		cerr << "error catched" << std::endl;
+		exit(3);
+	      }
+	      oldRowMask=rowMask;
+	    }
+	    if(lineMask[colMask]==msknodata_opt[0])
+	      valid=false;
+	  }
+	}
+
+	if(!valid)
+	  continue;
+
         //lookup corresponding row for irow in this file
         imgReader.geo2image(x,y,readCol,readRow);
         if(readCol<0||readCol>=imgReader.nrOfCol())
@@ -1014,6 +1241,11 @@ int main(int argc, char *argv[])
     progress=static_cast<float>(irow+1.0)/imgWriter.nrOfRow();
     pfnProgress(progress,pszMessage,pProgressArg);
   }
+  if(extent_opt.size()&&cut_opt.size()){
+    extentReader.close();
+  }
+  if(mask_opt.size())
+    maskReader.close();
   imgWriter.close();
 }
   
