@@ -1,5 +1,5 @@
 /**********************************************************************
-pkkalman.cc: program to kalman raster images: median, min/max, morphological, kalmaning
+pkkalman.cc: produce kalman filtered raster time series
 Copyright (C) 2008-2014 Pieter Kempeneers
 
 This file is part of pktools
@@ -27,6 +27,52 @@ along with pktools.  If not, see <http://www.gnu.org/licenses/>.
 #include "algorithms/StatFactory.h"
 #include "algorithms/ImgRegression.h"
 
+/******************************************************************************/
+/*! \page pkkalman pkkalman
+produce kalman filtered raster time series
+## SYNOPSIS
+
+<code>
+  
+</code>
+
+\section pkkalman_options Options
+ - use either `-short` or `--long` options (both `--long=value` and `--long value` are supported)
+ - short option `-h` shows basic options only, long option `--help` shows all options
+
+|short|long|type|default|description|
+|-----|----|----|-------|-----------|
+ | dir    | direction            | std::string | forward |direction to run model (forward\|backward\|smooth) | 
+ | mod    | model                | std::string |       |model input datasets, e.g., MODIS (use: -mod model1 -mod model2 etc. | 
+ | obs    | observation          | std::string |       |observation input datasets, e.g., landsat (use: -obs obs1 -obs obs2 etc. | 
+ | tmod   | tmodel               | int  |       |time sequence of model input. Sequence must have exact same length as model input. Leave empty to have default sequence 0,1,2,etc. | 
+ | tobs   | tobservation         | int  |       |time sequence of observation input. Sequence must have exact same length as observation input) | 
+ | a_srs  | a_srs                | std::string |       |Override the projection for the output file (leave blank to copy from input file, use epsg:3035 to use European projection and force to European grid | 
+ | ofw    | outputfw             | std::string |       |Output raster dataset for forward model | 
+ | obw    | outputbw             | std::string |       |Output raster dataset for backward model | 
+ | ofb    | outputfb             | std::string |       |Output raster dataset for smooth model | 
+ | modnodata | modnodata            | double | 0     |invalid value for model input | 
+ | obsnodata | obsnodata            | double | 0     |invalid value for observation input | 
+ | modoffset | modoffset            | double |       |offset used to read model input dataset (value=offset+scale*readValue | 
+ | modscale | modscale             | double |       |scale used to read model input dataset (value=offset+scale*readValue | 
+ | obsoffset | obsoffset            | double |       |offset used to read observation input dataset (value=offset+scale*readValue | 
+ | obsscale | obsscale             | double |       |scale used to read observation input dataset (value=offset+scale*readValue | 
+ | eps    | eps                  | double | 1e-05 |epsilon for non zero division | 
+ | um     | uncertmodel          | double | 2     |Multiply this value with std dev of first model image to obtain uncertainty of model | 
+ | uo     | uncertobs            | double | 0     |Uncertainty of valid observations | 
+ | w      | weight               | double |       |Set observation uncertainty as weighted difference between observation and model (use -w 0 to use a constant observation uncertainty, use -w value >> 1 to penalize low observation values with respect to model, use -w value << 0 to penalize a high observation values with respect to model | 
+ | dobs   | deltaobs             | double |       |Lower and upper thresholds for relative pixel differences (in percentage): (observation-model)/model. For instance to force the observation within a +/- 10 % interval, use: -dobs -10 -dobs 10 (equivalent to -dobs 10). Leave empty to always update on observation | 
+ | unodata | uncertnodata         | double | 10000 |Uncertainty in case of no-data values in observation | 
+ | rt     | regtime             | double | 1 |Set optional regression for sensor difference (model - observation). | 
+ | rs     | regsensor            | double | 1 |Set optional regression for sensor difference (model - observation). | 
+ | down   | down                 | int  |       |Downsampling factor for reading model data to calculate regression | 
+ | th     | threshold            | float | 0     |threshold for selecting samples (randomly). Provide probability in percentage (>0) or absolute (<0). | 
+ | win    | window               | unsigned short | 0     |window size for calculating regression (use 0 for global) | 
+ | co     | co                   | std::string |       |Creation option for output file. Multiple options can be specified. | 
+ | v      | verbose              | short | 0     |verbose mode when positive | 
+
+**/
+
 using namespace std;
 /*------------------
   Main procedure
@@ -53,8 +99,8 @@ int main(int argc,char **argv) {
   Optionpk<double> weight_opt("w", "weight", "Penalize outliers in measurement via weights. Use first weight to penalize small measurements wrt model and second weight to penalize large measurements wrt model");
   Optionpk<double> deltaObs_opt("dobs", "deltaobs", "Lower and upper thresholds for relative pixel differences (in percentage): (observation-model)/model. For instance to force the observation within a +/- 10 % interval, use: -dobs -10 -dobs 10 (equivalent to -dobs 10). Leave empty to always update on observation");
   Optionpk<double> uncertNodata_opt("unodata", "uncertnodata", "Uncertainty in case of no-data values in observation", 10000);
-  // Optionpk<double> regTime_opt("rt", "regtime", "Relative Weight for regression in time series", 1.0);
-  Optionpk<bool> regSensor_opt("rs", "regsensor", "Set optional regression for sensor difference (model - observation).", false);
+  Optionpk<double> regTime_opt("rt", "regtime", "Weight for regression in time series", 1.0);
+  Optionpk<double> regSensor_opt("rs", "regsensor", "Weight for regression model - measurement (model - observation).");
   Optionpk<int> down_opt("down", "down", "Downsampling factor for reading model data to calculate regression");
   Optionpk<float> threshold_opt("th", "threshold", "threshold for selecting samples (randomly). Provide probability in percentage (>0) or absolute (<0).", 0);
   Optionpk<int> minreg_opt("minreg", "minreg", "Minimum number of pixels to take into account for regression", 5, 2);
@@ -91,7 +137,7 @@ int main(int argc,char **argv) {
     weight_opt.retrieveOption(argc,argv);
     deltaObs_opt.retrieveOption(argc,argv);
     uncertNodata_opt.retrieveOption(argc,argv);
-    // regTime_opt.retrieveOption(argc,argv);
+    regTime_opt.retrieveOption(argc,argv);
     regSensor_opt.retrieveOption(argc,argv);
     down_opt.retrieveOption(argc,argv);
     threshold_opt.retrieveOption(argc,argv);
@@ -110,7 +156,7 @@ int main(int argc,char **argv) {
     exit(0);
   }
   if(!doProcess){
-    std::cout << "short option -h shows basic options only, use long option --help to show all options" << std::endl;
+    std::cerr << "short option -h shows basic options only, use long option --help to show all options" << std::endl;
     exit(0);//help was invoked, stop processing
   }
 
@@ -124,6 +170,10 @@ int main(int argc,char **argv) {
     weight_opt.push_back(weight_opt[0]);
   }
 
+  if(down_opt.empty()){
+    std::cerr << "short option -h shows basic options only, use long option --help to show all options" << std::endl;
+    exit(0);//help was invoked, stop processing
+  }
   try{
     ostringstream errorStream;
     if(model_opt.size()<2){
@@ -222,6 +272,9 @@ int main(int argc,char **argv) {
   }
   imgReaderObs.close();
 
+  if(regSensor_opt.empty())
+    regSensor_opt.push_back(1.0/down_opt[0]);
+  //hiero
   // ImgReaderGdal maskReader;
   // double colMask=0;
   // double rowMask=0;
@@ -268,17 +321,18 @@ int main(int argc,char **argv) {
 
   for(int tindex=0;tindex<tobservation_opt.size();++tindex){
     vector<int>::iterator modit;
-    modit=lower_bound(tmodel_opt.begin(),tmodel_opt.end(),tobservation_opt[tindex]);
-    int relpos=modit-tmodel_opt.begin();
-    // if(relpos<0)
-    //   relpos=0;
+    modit=upper_bound(tmodel_opt.begin(),tmodel_opt.end(),tobservation_opt[tindex]);
+    int relpos=modit-tmodel_opt.begin()-1;
+    assert(relpos>=0);//todo: for now, we assume model is available at time before first measurement
     relobsindex.push_back(relpos);
     if(verbose_opt[0])
-      cout << "tobservation_opt[tindex] " << tobservation_opt[tindex] << " " << relobsindex.back() << " " << observation_opt[tindex] << " " << model_opt[relpos] << endl;
+      cout << "observation " << tindex << ": " << "relative position in model time series is " << relpos << ", date of observation is (tobservation_opt[tindex]): " << tobservation_opt[tindex] << ", relobsindex.back(): " << relobsindex.back() << ", filename observation: " << observation_opt[tindex] << ", filename of corresponding model: " << model_opt[relpos] << endl;
     // if(verbose_opt[0])
     //   cout << "tobservation_opt[tindex] " << tobservation_opt[tindex] << " " << relobsindex.back() << endl;
   }
 
+  int ndigit=log(1.0*tmodel_opt.back())/log(10.0)+1;
+
   double geox=0;
   double geoy=0;
 
@@ -288,15 +342,21 @@ int main(int argc,char **argv) {
     obsindex=0;
     //initialization
     string output;
-    if(outputfw_opt.size()==model_opt.size())
+    if(outputfw_opt.size()==model_opt.size()){
       output=outputfw_opt[0];
+    }
     else{
       ostringstream outputstream;
-      outputstream << outputfw_opt[0] << "_" << tmodel_opt[0] << ".tif";
+      outputstream << outputfw_opt[0] << "_";
+      outputstream << setfill('0') << setw(ndigit) << tmodel_opt[0];
+      outputstream << ".tif";
+      //test
+      // outputstream << outputfw_opt[0] << "_" << tmodel_opt[0] << ".tif";
       output=outputstream.str();
     }
     if(verbose_opt[0])
       cout << "Opening image " << output << " for writing " << endl;
+
     imgWriterEst.open(output,ncol,nrow,2,GDT_Float32,imageType,option_opt);
     imgWriterEst.setProjectionProj4(projection_opt[0]);
     imgWriterEst.setGeoTransform(geotransform);
@@ -344,7 +404,10 @@ int main(int argc,char **argv) {
 	// vector<double> lineMask;
 	imgWriterEst.image2geo(0,irow,geox,geoy);
 	imgReaderModel1.geo2image(geox,geoy,modCol,modRow);
-	assert(modRow>=0&&modRow<imgReaderModel1.nrOfRow());
+	if(modRow<0||modRow>=imgReaderModel1.nrOfRow()){
+	  cerr << "Error: geo coordinates (" << geox << "," << geoy << ") not covered in model image " << imgReaderModel1.getFileName() << endl;
+	  assert(modRow>=0&&modRow<imgReaderModel1.nrOfRow());
+	}
 	try{
 	  imgReaderModel1.readData(estReadBuffer,GDT_Float64,modRow);
 	  //simple nearest neighbor
@@ -360,6 +423,7 @@ int main(int argc,char **argv) {
 	      uncertWriteBuffer[icol]=uncertNodata_opt[0];
 	    }
 	    else{
+	      //todo: should take into account regression model-obs...
 	      estWriteBuffer[icol]=modValue;
 	      uncertWriteBuffer[icol]=uncertModel_opt[0]*stdDev;
 	    }
@@ -375,9 +439,9 @@ int main(int argc,char **argv) {
 	}
       }
     }
-    else{//we have a measurement at time 0
+    else{//we have a measurement
       if(verbose_opt[0])
-	cout << "we have an observation at time 0" << endl;
+	cout << "we have a measurement at initial time" << endl;
       imgReaderObs.open(observation_opt[0]);
       imgReaderObs.getGeoTransform(geotransform);
       imgReaderObs.setNoData(obsnodata_opt);
@@ -386,8 +450,13 @@ int main(int argc,char **argv) {
       if(obsscale_opt.size())
 	imgReaderObs.setScale(obsscale_opt[0]);
 
-      if(regSensor_opt[0])
-	errObs=imgreg.getRMSE(imgReaderModel1,imgReaderObs,c0obs,c1obs,0,0,verbose_opt[0]);
+      if(regSensor_opt[0]>0){
+	errObs=regSensor_opt[0]*imgreg.getRMSE(imgReaderModel1,imgReaderObs,c0obs,c1obs,0,0,verbose_opt[0]);
+	if(errObs<0){
+	  c0obs=0;
+	  c1obs=1;
+	}
+      }
       else{
 	c0obs=0;
 	c1obs=1;
@@ -431,21 +500,23 @@ int main(int argc,char **argv) {
 	  }
 	  else{//model is valid: calculate estimate from model
 	    double errMod=uncertModel_opt[0]*stdDev;
-	    double certNorm=(errMod*errMod+errObs*errObs);
-	    double certMod=errObs*errObs/certNorm;
-	    double certObs=errMod*errMod/certNorm;
-	    double regTime=0;
-	    double regSensor=(c0obs+c1obs*modValue)*certObs;
-	    estWriteBuffer[icol]=regTime+regSensor;
-	    double totalUncertainty=0;
-	    if(errMod<eps_opt[0])
-	      totalUncertainty=errObs;
-	    else if(errObs<eps_opt[0])
-	      totalUncertainty=errMod;
-	    else{
-	      totalUncertainty=1.0/errMod/errMod+1/errObs/errObs;
-	      totalUncertainty=sqrt(1.0/totalUncertainty);
-	    }
+	    errMod*=regTime_opt[0];
+	    // double certNorm=(errMod*errMod+errObs*errObs);
+	    // double certMod=errObs*errObs/certNorm;
+	    // double certObs=errMod*errMod/certNorm;
+	    // double regTime=0;
+	    // double regSensor=(c0obs+c1obs*modValue)*certMod;
+	    // estWriteBuffer[icol]=regTime+regSensor;
+	    estWriteBuffer[icol]=modValue;
+	    double totalUncertainty=errMod;
+	    // if(errMod<eps_opt[0])
+	    //   totalUncertainty=errObs;
+	    // else if(errObs<eps_opt[0])
+	    //   totalUncertainty=errMod;
+	    // else{
+	    //   totalUncertainty=1.0/errMod/errMod+1/errObs/errObs;
+	    //   totalUncertainty=sqrt(1.0/totalUncertainty);
+	    // }
 	    uncertWriteBuffer[icol]=totalUncertainty;//in case observation is not valid
 	  }
 	  // uncertWriteBuffer[icol]+=uncertReadBuffer[icol];
@@ -520,8 +591,10 @@ int main(int argc,char **argv) {
 	output=outputfw_opt[modindex];
       else{
 	ostringstream outputstream;
-	outputstream << outputfw_opt[0] << "_" << tmodel_opt[modindex] << ".tif";
-	// outputstream << output_opt[0] << "_" << modindex+1 << ".tif";
+	outputstream << outputfw_opt[0] << "_";
+	outputstream << setfill('0') << setw(ndigit) << tmodel_opt[modindex];
+	outputstream << ".tif";
+	// outputstream << outputfw_opt[0] << "_" << tmodel_opt[modindex] << ".tif";
 	output=outputstream.str();
       }
     
@@ -554,6 +627,8 @@ int main(int argc,char **argv) {
 	cout << "Calculating regression for " << imgReaderModel1.getFileName() << " " << imgReaderModel2.getFileName() << endl;
       
       double errMod=imgreg.getRMSE(imgReaderModel1,imgReaderModel2,c0modGlobal,c1modGlobal,0,0);
+      errMod*=regTime_opt[0];
+
       // double errMod=imgreg.getRMSE(imgReaderModel1,imgReaderModel2,c0modGlobal,c1modGlobal,verbose_opt[0]);
       if(verbose_opt[0])
 	cout << "c0modGlobal, c1modGlobal: " << c0modGlobal << ", " << c1modGlobal << endl;
@@ -576,15 +651,20 @@ int main(int argc,char **argv) {
 	//calculate regression between model and observation
 	if(verbose_opt[0])
 	  cout << "Calculating regression for " << imgReaderModel2.getFileName() << " " << imgReaderObs.getFileName() << endl;
-	if(regSensor_opt[0])
-	  errObs=imgreg.getRMSE(imgReaderModel2,imgReaderObs,c0obs,c1obs,0,0,verbose_opt[0]);
+	if(regSensor_opt[0]>0){
+	  errObs=regSensor_opt[0]*imgreg.getRMSE(imgReaderModel2,imgReaderObs,c0obs,c1obs,0,0,verbose_opt[0]);
+	  if(errObs<0){
+	    c0obs=0;
+	    c1obs=1;
+	  }
+	}
 	else{
 	  c0obs=0;
 	  c1obs=1;
 	  errObs=0;
 	}
 	if(verbose_opt[0])
-	  cout << "c0obs, c1obs: " << c0obs << ", " << c1obs << endl;
+	  cout << "c0obs, c1obs, errObs: " << c0obs << ", " << c1obs << ", " << errObs << endl;
       }
       //prediction (also to fill cloudy pixels in update mode)
       string input;
@@ -592,9 +672,14 @@ int main(int argc,char **argv) {
 	input=outputfw_opt[modindex-1];
       else{
 	ostringstream outputstream;
-	outputstream << outputfw_opt[0] << "_" << tmodel_opt[modindex-1] << ".tif";
+	outputstream << outputfw_opt[0] << "_";
+	outputstream << setfill('0') << setw(ndigit) << tmodel_opt[modindex-1];
+	outputstream << ".tif";
+	// outputstream << outputfw_opt[0] << "_" << tmodel_opt[modindex-1] << ".tif";
 	input=outputstream.str();
       }
+      if(verbose_opt[0])
+	cout << "opening " << input << endl;
       ImgReaderGdal imgReaderEst(input);
       imgReaderEst.setNoData(obsnodata_opt);
       if(obsoffset_opt.size())
@@ -617,16 +702,19 @@ int main(int argc,char **argv) {
       vector<double> uncertWriteBuffer(ncol);
       // vector<double> lineMask;
 
-      //initialize obsLineVector
-      assert(down_opt[0]%2);//window size must be odd 
-      for(int iline=-down_opt[0]/2+1;iline<down_opt[0]/2+1;++iline){
-	if(iline<0)//replicate line 0
-	  imgReaderObs.readData(obsLineVector[iline+down_opt[0]/2],GDT_Float64,0,0);
-	else
-	  imgReaderObs.readData(obsLineVector[iline+down_opt[0]/2],GDT_Float64,iline,0);
+      //initialize obsLineVector if update
+      if(update){
+	if(verbose_opt[0])
+	  cout << "initialize obsLineVector" << endl;
+	assert(down_opt[0]%2);//window size must be odd 
+	for(int iline=-down_opt[0]/2;iline<down_opt[0]/2+1;++iline){
+	  if(iline<0)//replicate line 0
+	    imgReaderObs.readData(obsLineVector[iline+down_opt[0]/2],GDT_Float64,0,0);
+	  else
+	    imgReaderObs.readData(obsLineVector[iline+down_opt[0]/2],GDT_Float64,iline,0);
+	}
       }
       for(int irow=0;irow<imgWriterEst.nrOfRow();++irow){
-	assert(irow<imgReaderEst.nrOfRow());
 	//do not read from imgReaderObs, because we read entire window for each pixel...
 	imgReaderEst.readData(estReadBuffer,GDT_Float64,irow,0);
 	imgReaderEst.readData(uncertReadBuffer,GDT_Float64,irow,1);
@@ -725,8 +813,10 @@ int main(int argc,char **argv) {
 		  ++it2;
 		}
 	      }
-	      if(model1buffer.size()>minreg_opt[0]&&model2buffer.size()>minreg_opt[0])
+	      if(model1buffer.size()>minreg_opt[0]&&model2buffer.size()>minreg_opt[0]){
 		errMod=stat.linear_regression_err(model1buffer,model2buffer,c0mod,c1mod);
+		errMod*=regTime_opt[0];
+	      }
 	      else{//use global regression...
 		c0mod=c0modGlobal;
 		c1mod=c1modGlobal;
@@ -739,11 +829,18 @@ int main(int argc,char **argv) {
 	    double certNorm=(errMod*errMod+errObs*errObs);
 	    double certMod=errObs*errObs/certNorm;
 	    double certObs=errMod*errMod/certNorm;
-	    double regTime=(c0mod+c1mod*estValue)*certMod;
-
+	    double regTime=(c0mod+c1mod*estValue)*certObs;
+	    double regSensor=(c0obs+c1obs*modValue)*certMod;
 	    // double regSensor=(c0obs+c1obs*estValue)*certObs;
-	    double regSensor=(c0obs+c1obs*modValue)*certObs;
 	    estWriteBuffer[icol]=regTime+regSensor;
+	    //test
+	    // if(regTime<regSensor){
+	    //   cout << "regTime = (" << c0mod << "+" << c1mod << "*" << estValue << ")*" << certObs << " = " << regTime << endl;
+	    //   cout << "regSensor = (" << c0obs << "+" << c1obs << "*" << modValue << ")*" << certMod << " = " << regSensor << endl;
+	    //   assert(regTime+regSensor>0);
+	    //   assert(regTime+regSensor<=1);
+	    // }
+
 	    double totalUncertainty=0;
 	    if(errMod<eps_opt[0])
 	      totalUncertainty=errObs;
@@ -824,7 +921,10 @@ int main(int argc,char **argv) {
       output=outputbw_opt.back();
     else{
       ostringstream outputstream;
-      outputstream << outputbw_opt[0] << "_" << tmodel_opt.back() << ".tif";
+      outputstream << outputbw_opt[0] << "_";
+      outputstream << setfill('0') << setw(ndigit) << tmodel_opt.back();
+      outputstream << ".tif";
+      // outputstream << outputbw_opt[0] << "_" << tmodel_opt.back() << ".tif";
       output=outputstream.str();
     }
     if(verbose_opt[0])
@@ -907,9 +1007,9 @@ int main(int argc,char **argv) {
 	}
       }
     }
-    else{//we have an observation at end time
+    else{//we have an measurement at end time
       if(verbose_opt[0])
-	cout << "we have an observation at end time" << endl;
+	cout << "we have an measurement at end time" << endl;
       imgReaderObs.open(observation_opt.back());
       imgReaderObs.getGeoTransform(geotransform);
       imgReaderObs.setNoData(obsnodata_opt);
@@ -918,8 +1018,13 @@ int main(int argc,char **argv) {
       if(obsscale_opt.size())
 	imgReaderObs.setScale(obsscale_opt[0]);
       
-      if(regSensor_opt[0])
-	errObs=imgreg.getRMSE(imgReaderModel1,imgReaderObs,c0obs,c1obs,0,0,verbose_opt[0]);
+      if(regSensor_opt[0]>0){
+	errObs=regSensor_opt[0]*imgreg.getRMSE(imgReaderModel1,imgReaderObs,c0obs,c1obs,0,0,verbose_opt[0]);
+	if(errObs<0){
+	  c0obs=0;
+	  c1obs=1;
+	}
+      }
       else{
 	c0obs=0;
 	c1obs=1;
@@ -963,21 +1068,23 @@ int main(int argc,char **argv) {
 	  }
 	  else{//model is valid: calculate estimate from model
 	    double errMod=uncertModel_opt[0]*stdDev;
-	    double certNorm=(errMod*errMod+errObs*errObs);
-	    double certMod=errObs*errObs/certNorm;
-	    double certObs=errMod*errMod/certNorm;
-	    double regTime=0;
-	    double regSensor=(c0obs+c1obs*modValue)*certObs;
-	    estWriteBuffer[icol]=regTime+regSensor;
-	    double totalUncertainty=0;
-	    if(errMod<eps_opt[0])
-	      totalUncertainty=errObs;
-	    else if(errObs<eps_opt[0])
-	      totalUncertainty=errMod;
-	    else{
-	      totalUncertainty=1.0/errMod/errMod+1/errObs/errObs;
-	      totalUncertainty=sqrt(1.0/totalUncertainty);
-	    }
+	    errMod*=regTime_opt[0];
+	    // double certNorm=(errMod*errMod+errObs*errObs);
+	    // double certMod=errObs*errObs/certNorm;
+	    // double certObs=errMod*errMod/certNorm;
+	    // double regTime=0;
+	    // double regSensor=(c0obs+c1obs*modValue)*certMod;
+	    // estWriteBuffer[icol]=regTime+regSensor;
+	    estWriteBuffer[icol]=modValue;
+	    double totalUncertainty=errMod;
+	    // if(errMod<eps_opt[0])
+	    //   totalUncertainty=errObs;
+	    // else if(errObs<eps_opt[0])
+	    //   totalUncertainty=errMod;
+	    // else{
+	    //   totalUncertainty=1.0/errMod/errMod+1/errObs/errObs;
+	    //   totalUncertainty=sqrt(1.0/totalUncertainty);
+	    // }
 	    uncertWriteBuffer[icol]=totalUncertainty;//in case observation is not valid
 	  }
 	  //measurement update
@@ -1051,8 +1158,10 @@ int main(int argc,char **argv) {
 	output=outputbw_opt[modindex];
       else{
 	ostringstream outputstream;
-	outputstream << outputbw_opt[0] << "_" << tmodel_opt[modindex] << ".tif";
-	// outputstream << output_opt[0] << "_" << modindex+1 << ".tif";
+	outputstream << outputbw_opt[0] << "_";
+	outputstream << setfill('0') << setw(ndigit) << tmodel_opt[modindex];
+	outputstream << ".tif";
+	// outputstream << outputbw_opt[0] << "_" << tmodel_opt[modindex] << ".tif";
 	output=outputstream.str();
       }
 
@@ -1085,6 +1194,8 @@ int main(int argc,char **argv) {
 	cout << "Calculating regression for " << imgReaderModel1.getFileName() << " " << imgReaderModel2.getFileName() << endl;
 
       double errMod=imgreg.getRMSE(imgReaderModel1,imgReaderModel2,c0modGlobal,c1modGlobal,0,0);
+      errMod*=regTime_opt[0];
+
       // double errMod=imgreg.getRMSE(imgReaderModel1,imgReaderModel2,c0modGlobal,c1modGlobal,verbose_opt[0]);
       if(verbose_opt[0])
 	cout << "c0modGlobal, c1modGlobal: " << c0modGlobal << ", " << c1modGlobal << endl;
@@ -1107,8 +1218,13 @@ int main(int argc,char **argv) {
 	//calculate regression between model and observation
 	if(verbose_opt[0])
 	  cout << "Calculating regression for " << imgReaderModel2.getFileName() << " " << imgReaderObs.getFileName() << endl;
-	if(regSensor_opt[0])
-	  errObs=imgreg.getRMSE(imgReaderModel2,imgReaderObs,c0obs,c1obs,0,0,verbose_opt[0]);
+	if(regSensor_opt[0]>0){
+	  errObs=regSensor_opt[0]*imgreg.getRMSE(imgReaderModel2,imgReaderObs,c0obs,c1obs,0,0,verbose_opt[0]);
+	  if(errObs<0){
+	    c0obs=0;
+	    c1obs=1;
+	  }
+	}
 	else{
 	  c0obs=0;
 	  c1obs=1;
@@ -1123,7 +1239,10 @@ int main(int argc,char **argv) {
 	input=outputbw_opt[modindex+1];
       else{
 	ostringstream outputstream;
-	outputstream << outputbw_opt[0] << "_" << tmodel_opt[modindex+1] << ".tif";
+	outputstream << outputbw_opt[0] << "_";
+	outputstream << setfill('0') << setw(ndigit) << tmodel_opt[modindex+1];
+	outputstream << ".tif";
+	// outputstream << outputbw_opt[0] << "_" << tmodel_opt[modindex+1] << ".tif";
 	input=outputstream.str();
       }
       ImgReaderGdal imgReaderEst(input);
@@ -1149,12 +1268,14 @@ int main(int argc,char **argv) {
       // vector<double> lineMask;
 
       //initialize obsLineVector
-      assert(down_opt[0]%2);//window size must be odd 
-      for(int iline=-down_opt[0]/2+1;iline<down_opt[0]/2+1;++iline){
-	if(iline<0)//replicate line 0
-	  imgReaderObs.readData(obsLineVector[iline+down_opt[0]/2],GDT_Float64,0,0);
-	else
-	  imgReaderObs.readData(obsLineVector[iline+down_opt[0]/2],GDT_Float64,iline,0);
+      if(update){
+	assert(down_opt[0]%2);//window size must be odd 
+	for(int iline=-down_opt[0]/2;iline<down_opt[0]/2+1;++iline){
+	  if(iline<0)//replicate line 0
+	    imgReaderObs.readData(obsLineVector[iline+down_opt[0]/2],GDT_Float64,0,0);
+	  else
+	    imgReaderObs.readData(obsLineVector[iline+down_opt[0]/2],GDT_Float64,iline,0);
+	}
       }
       for(int irow=0;irow<imgWriterEst.nrOfRow();++irow){
 	assert(irow<imgReaderEst.nrOfRow());
@@ -1256,8 +1377,10 @@ int main(int argc,char **argv) {
 		  ++it2;
 		}
 	      }
-	      if(model1buffer.size()>minreg_opt[0]&&model2buffer.size()>minreg_opt[0])
+	      if(model1buffer.size()>minreg_opt[0]&&model2buffer.size()>minreg_opt[0]){
 		errMod=stat.linear_regression_err(model1buffer,model2buffer,c0mod,c1mod);
+		errMod*=regTime_opt[0];
+	      }
 	      else{//use global regression...
 		c0mod=c0modGlobal;
 		c1mod=c1modGlobal;
@@ -1270,10 +1393,10 @@ int main(int argc,char **argv) {
 	    double certNorm=(errMod*errMod+errObs*errObs);
 	    double certMod=errObs*errObs/certNorm;
 	    double certObs=errMod*errMod/certNorm;
-	    double regTime=(c0mod+c1mod*estValue)*certMod;
+	    double regTime=(c0mod+c1mod*estValue)*certObs;
 
 	    // double regSensor=(c0obs+c1obs*estValue)*certObs;
-	    double regSensor=(c0obs+c1obs*modValue)*certObs;
+	    double regSensor=(c0obs+c1obs*modValue)*certMod;
 	    estWriteBuffer[icol]=regTime+regSensor;
 	    double totalUncertainty=0;
 	    if(errMod<eps_opt[0])
@@ -1362,7 +1485,10 @@ int main(int argc,char **argv) {
 	output=outputfb_opt[modindex];
       else{
 	ostringstream outputstream;
-	outputstream << outputfb_opt[0] << "_" << tmodel_opt[modindex] << ".tif";
+	outputstream << outputfb_opt[0] << "_";
+	outputstream << setfill('0') << setw(ndigit) << tmodel_opt[modindex];
+	outputstream << ".tif";
+	// outputstream << outputfb_opt[0] << "_" << tmodel_opt[modindex] << ".tif";
 	output=outputstream.str();
       }
     
@@ -1381,14 +1507,20 @@ int main(int argc,char **argv) {
 	inputfw=outputfw_opt[modindex];
       else{
 	ostringstream outputstream;
-	outputstream << outputfw_opt[0] << "_" << tmodel_opt[modindex] << ".tif";
+	outputstream << outputfw_opt[0] << "_";
+	outputstream << setfill('0') << setw(ndigit) << tmodel_opt[modindex];
+	outputstream << ".tif";
+	// outputstream << outputfw_opt[0] << "_" << tmodel_opt[modindex] << ".tif";
 	inputfw=outputstream.str();
       }
       if(outputbw_opt.size()==model_opt.size())
 	inputbw=outputbw_opt[modindex];
       else{
 	ostringstream outputstream;
-	outputstream << outputbw_opt[0] << "_" << tmodel_opt[modindex] << ".tif";
+	outputstream << outputbw_opt[0] << "_";
+	outputstream << setfill('0') << setw(ndigit) << tmodel_opt[modindex];
+	outputstream << ".tif";
+	// outputstream << outputbw_opt[0] << "_" << tmodel_opt[modindex] << ".tif";
 	inputbw=outputstream.str();
       }
       ImgReaderGdal imgReaderForward(inputfw);
