@@ -25,6 +25,44 @@ along with pktools.  If not, see <http://www.gnu.org/licenses/>.
 #include "imageclasses/ImgReaderGdal.h"
 #include "imageclasses/ImgWriterGdal.h"
 
+/******************************************************************************/
+/*! \page pkreclass pkreclass
+ program to replace pixel values in raster image
+## SYNOPSIS
+
+<code>
+  Usage: pkreclass -i input [-c from -r to]* -o output
+</code>
+
+\section pkreclass_options Options
+ - use either `-short` or `--long` options (both `--long=value` and `--long value` are supported)
+ - short option `-h` shows basic options only, long option `--help` shows all options
+|short|long|type|default|description|
+|-----|----|----|-------|-----------|
+ | i      | input                | std::string |       |Input image | 
+ | m      | mask                 | std::string |       |Mask image(s) | 
+ | msknodata | msknodata            | unsigned short | 1     |Mask value(s) where image has nodata. Use one value for each mask, or multiple values for a single mask. | 
+ | nodata | nodata               | int  | 0     |nodata value to put in image if not valid (0) | 
+ | code   | code                 | std::string |       |Recode text file (2 colums: from to) | 
+ | c      | class                | std::string |       |list of classes to reclass (in combination with reclass option) | 
+ | r      | reclass              | std::string |       |list of recoded classes (in combination with class option) | 
+ | ct     | ct                   | std::string |       |color table (file with 5 columns: id R G B ALFA (0: transparent, 255: solid) | 
+ | o      | output               | std::string |       |Output mask file | 
+ | ot     | otype                | std::string |       |Data type for output image ({Byte/Int16/UInt16/UInt32/Int32/Float32/Float64/CInt16/CInt32/CFloat32/CFloat64}). Empty string: inherit type from input image | 
+ | b      | band                 | unsigned short | 0     |band index(es) to replace (other bands are copied to output) | 
+ | n      | fname                | std::string | label |field name of the shape file to be replaced | 
+ | co     | co                   | std::string |       |Creation option for output file. Multiple options can be specified. | 
+ | d      | description          | std::string |       |Set image description | 
+ | v      | verbose              | short | 0     |verbose | 
+
+Usage: pkreclass -i input [-c from -r to]* -o output
+
+
+Examples
+========
+Some examples how to use pkreclass can be found \ref examples_pkreclass "here"
+**/
+
 using namespace std;
 
 int main(int argc, char *argv[])
