@@ -33,6 +33,87 @@ along with pktools.  If not, see <http://www.gnu.org/licenses/>.
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+/******************************************************************************/
+/*! \page pksvm pksvm
+ classify raster image using Support Vector Machine
+## SYNOPSIS
+
+<code>
+  Usage: pksvm -t training [-i input -o output] [-cv value]
+</code>
+
+<code>
+
+  Options: [-tln layer]* [-c name -r value]* [-of GDALformat|-f OGRformat] [-co NAME=VALUE]* [-ct filename] [-label attribute] [-prior value]* [-g gamma] [-cc cost] [-m filename [-msknodata value]*] [-nodata value]
+
+  Advanced options:
+       [-b band] [-sband band -eband band]* [-bal size]* [-min] [-bag value] [-bs value] [-comb rule] [-cb filename] [-prob filename] [-pim priorimage] [--offset value] [--scale value] [-svmt type] [-kt type] [-kd value]  [-c0 value] [-nu value] [-eloss value] [-cache value] [-etol value] [-shrink]
+</code>
+
+\section pksvm_description Description
+
+The utility pksvm implements a support vector machine (SVM) to solve a supervised classification problem. The implementation is based on the open source C++ library libSVM (http://www.csie.ntu.edu.tw/~cjlin/libsvm). 
+Both raster and vector files are supported as input. The output will contain the classification result, either in raster or vector format, corresponding to the format of the input. A training sample must be provided as an OGR vector dataset that contains the class labels and the features for each training point. The point locations are not considered in the training step. You can use the same training sample for classifying different images, provided the number of bands of the images are identical. Use the utility \ref pkextract "pkextract" to create a suitable training sample, based on a sample of points or polygons. For raster output maps you can attach a color table using the option -ct.
+
+\section pksvm_options Options
+ - use either `-short` or `--long` options (both `--long=value` and `--long value` are supported)
+ - short option `-h` shows basic options only, long option `--help` shows all options
+|short|long|type|default|description|
+|-----|----|----|-------|-----------|
+ | t      | training             | std::string |       |Training vector file. A single vector file contains all training features (must be set as: b0, b1, b2,...) for all classes (class numbers identified by label option). Use multiple training files for bootstrap aggregation (alternative to the bag and bsize options, where a random subset is taken from a single training file) | 
+ | i      | input                | std::string |       |input image | 
+ | o      | output               | std::string |       |Output classification image | 
+ | cv     | cv                   | unsigned short | 0     |N-fold cross validation mode | 
+ | cmf    | cmf                  | std::string | ascii |Format for confusion matrix (ascii or latex) | 
+ | tln    | tln                  | std::string |       |Training layer name(s) | 
+ | c      | class                | std::string |       |List of class names. | 
+ | r      | reclass              | short |       |List of class values (use same order as in class opt). | 
+ | of     | oformat              | std::string |       |Output image format (see also gdal_translate). Empty string: inherit from input image | 
+ | f      | f                    | std::string | SQLite |Output ogr format for active training sample | 
+ | co     | co                   | std::string |       |Creation option for output file. Multiple options can be specified. | 
+ | ct     | ct                   | std::string |       |Color table in ASCII format having 5 columns: id R G B ALFA (0: transparent, 255: solid) | 
+ | label  | label                | std::string | label |Attribute name for class label in training vector file. | 
+ | prior  | prior                | double | 0     |Prior probabilities for each class (e.g., -p 0.3 -p 0.3 -p 0.2 ). Used for input only (ignored for cross validation) | 
+ | g      | gamma                | float | 1     |Gamma in kernel function | 
+ | cc     | ccost                | float | 1000  |The parameter C of C_SVC, epsilon_SVR, and nu_SVR | 
+ | m      | mask                 | std::string |       |Only classify within specified mask (vector or raster). For raster mask, set nodata values with the option msknodata. | 
+ | msknodata | msknodata            | short | 0     |Mask value(s) not to consider for classification (use negative values if only these values should be taken into account). Values will be taken over in classification image. | 
+ | nodata | nodata               | unsigned short | 0     |Nodata value to put where image is masked as nodata | 
+ | b      | band                 | short |       |Band index (starting from 0, either use band option or use start to end) | 
+ | sband  | startband            | unsigned short |      |Start band sequence number | 
+ | eband  | endband              | unsigned short |      |End band sequence number   | 
+ | bal    | balance              | unsigned int | 0     |Balance the input data to this number of samples for each class | 
+ | min    | min                  | int  | 0     |If number of training pixels is less then min, do not take this class into account (0: consider all classes) | 
+ | bag    | bag                  | unsigned short | 1     |Number of bootstrap aggregations | 
+ | bagsize | bagsize              | int  | 100   |Percentage of features used from available training features for each bootstrap aggregation (one size for all classes, or a different size for each class respectively | 
+ | comb   | comb                 | unsigned short | 0     |How to combine bootstrap aggregation classifiers (0: sum rule, 1: product rule, 2: max rule). Also used to aggregate classes with rc option. | 
+ | cb     | classbag             | std::string |       |Output for each individual bootstrap aggregation | 
+ | prob   | prob                 | std::string |       |Probability image. | 
+ | pim    | priorimg             | std::string |       |Prior probability image (multi-band img with band for each class | 
+ | offset | offset               | double | 0     |Offset value for each spectral band input features: refl[band]=(DN[band]-offset[band])/scale[band] | 
+ | scale  | scale                | double | 0     |Scale value for each spectral band input features: refl=(DN[band]-offset[band])/scale[band] (use 0 if scale min and max in each band to -1.0 and 1.0) | 
+ | svmt   | svmtype              | std::string | C_SVC |Type of SVM (C_SVC, nu_SVC,one_class, epsilon_SVR, nu_SVR) | 
+ | kt     | kerneltype           | std::string | radial |Type of kernel function (linear,polynomial,radial,sigmoid)  | 
+ | kd     | kd                   | unsigned short | 3     |Degree in kernel function | 
+ | c0     | coef0                | float | 0     |Coef0 in kernel function | 
+ | nu     | nu                   | float | 0.5   |The parameter nu of nu_SVC, one_class SVM, and nu_SVR | 
+ | eloss  | eloss                | float | 0.1   |The epsilon in loss function of epsilon_SVR | 
+ | cache  | cache                | int  | 100   |Cache memory size in MB | 
+ | etol   | etol                 | float | 0.001 |The tolerance of termination criterion | 
+ | shrink | shrink               | bool | false |Whether to use the shrinking heuristics | 
+ | pe     | probest              | bool | true  |Whether to train a SVC or SVR model for probability estimates | 
+ | entropy | entropy              | std::string |       |Entropy image (measure for uncertainty of classifier output | 
+ | active | active               | std::string |       |Ogr output for active training sample. | 
+ | na     | nactive              | unsigned int | 1     |Number of active training points | 
+ | random | random               | bool | true  |Randomize training data for balancing and bagging | 
+
+Usage: pksvm -t training [-i input -o output] [-cv value]
+
+
+Examples
+========
+Some examples how to use pksvm can be found \ref examples_pksvm "here"
+**/
 
 namespace svm{
   enum SVM_TYPE {C_SVC=0, nu_SVC=1,one_class=2, epsilon_SVR=3, nu_SVR=4};
@@ -55,14 +136,15 @@ int main(int argc, char *argv[])
   Optionpk<unsigned int> balance_opt("bal", "balance", "Balance the input data to this number of samples for each class", 0);
   Optionpk<bool> random_opt("random", "random", "Randomize training data for balancing and bagging", true, 2);
   Optionpk<int> minSize_opt("min", "min", "If number of training pixels is less then min, do not take this class into account (0: consider all classes)", 0);
-  Optionpk<short> band_opt("b", "band", "Band index (starting from 0, either use band option or use start to end)");
-  Optionpk<double> bstart_opt("s", "start", "Start band sequence number",0); 
-  Optionpk<double> bend_opt("e", "end", "End band sequence number (set to 0 to include all bands)", 0); 
-  Optionpk<double> offset_opt("\0", "offset", "Offset value for each spectral band input features: refl[band]=(DN[band]-offset[band])/scale[band]", 0.0);
-  Optionpk<double> scale_opt("\0", "scale", "Scale value for each spectral band input features: refl=(DN[band]-offset[band])/scale[band] (use 0 if scale min and max in each band to -1.0 and 1.0)", 0.0);
+  Optionpk<unsigned short> band_opt("b", "band", "Band index (starting from 0, either use band option or use start to end)");
+  Optionpk<unsigned short> bstart_opt("sband", "startband", "Start band sequence number"); 
+  Optionpk<unsigned short> bend_opt("eband", "endband", "End band sequence number"); 
+  Optionpk<double> offset_opt("offset", "offset", "Offset value for each spectral band input features: refl[band]=(DN[band]-offset[band])/scale[band]", 0.0);
+  Optionpk<double> scale_opt("scale", "scale", "Scale value for each spectral band input features: refl=(DN[band]-offset[band])/scale[band] (use 0 if scale min and max in each band to -1.0 and 1.0)", 0.0);
   Optionpk<double> priors_opt("prior", "prior", "Prior probabilities for each class (e.g., -p 0.3 -p 0.3 -p 0.2 ). Used for input only (ignored for cross validation)", 0.0); 
   Optionpk<string> priorimg_opt("pim", "priorimg", "Prior probability image (multi-band img with band for each class","",2); 
   Optionpk<unsigned short> cv_opt("cv", "cv", "N-fold cross validation mode",0);
+  Optionpk<string> cmformat_opt("cmf","cmf","Format for confusion matrix (ascii or latex)","ascii");
   Optionpk<std::string> svm_type_opt("svmt", "svmtype", "Type of SVM (C_SVC, nu_SVC,one_class, epsilon_SVR, nu_SVR)","C_SVC");
   Optionpk<std::string> kernel_type_opt("kt", "kerneltype", "Type of kernel function (linear,polynomial,radial,sigmoid) ","radial");
   Optionpk<unsigned short> kernel_degree_opt("kd", "kd", "Degree in kernel function",3);
@@ -80,7 +162,7 @@ int main(int argc, char *argv[])
   Optionpk<unsigned short> bag_opt("bag", "bag", "Number of bootstrap aggregations", 1);
   Optionpk<int> bagSize_opt("bagsize", "bagsize", "Percentage of features used from available training features for each bootstrap aggregation (one size for all classes, or a different size for each class respectively", 100);
   Optionpk<string> classBag_opt("cb", "classbag", "Output for each individual bootstrap aggregation");
-  Optionpk<string> mask_opt("m", "mask", "Use the first band of the specified file as a validity mask. Nodata values can be set with the option msknodata.");
+  Optionpk<string> mask_opt("m", "mask", "Only classify within specified mask (vector or raster). For raster mask, set nodata values with the option msknodata.");
   Optionpk<short> msknodata_opt("msknodata", "msknodata", "Mask value(s) not to consider for classification (use negative values if only these values should be taken into account). Values will be taken over in classification image.", 0);
   Optionpk<unsigned short> nodata_opt("nodata", "nodata", "Nodata value to put where image is masked as nodata", 0);
   Optionpk<string> output_opt("o", "output", "Output classification image"); 
@@ -122,15 +204,17 @@ int main(int argc, char *argv[])
   entropy_opt.setHide(1);
   active_opt.setHide(1);
   nactive_opt.setHide(1);
-  verbose_opt.setHide(1);
   random_opt.setHide(1);
 
+  verbose_opt.setHide(2);
+
   bool doProcess;//stop process when program was invoked with help option (-h --help)
   try{
     doProcess=training_opt.retrieveOption(argc,argv);
     input_opt.retrieveOption(argc,argv);
     output_opt.retrieveOption(argc,argv);
     cv_opt.retrieveOption(argc,argv);
+    cmformat_opt.retrieveOption(argc,argv);
     tlayer_opt.retrieveOption(argc,argv);
     classname_opt.retrieveOption(argc,argv);
     classvalue_opt.retrieveOption(argc,argv);
@@ -226,6 +310,30 @@ int main(int argc, char *argv[])
   if(verbose_opt[0]>=1)
     std::cout << "number of bootstrap aggregations: " << nbag << std::endl;
 
+  ImgReaderOgr extentReader;
+  OGRLayer  *readLayer;
+
+  double ulx=0;
+  double uly=0;
+  double lrx=0;
+  double lry=0;
+
+  bool maskIsVector=false;
+  if(mask_opt.size()){
+    try{
+      extentReader.open(mask_opt[0]);
+      maskIsVector=true;
+      readLayer = extentReader.getDataSource()->GetLayer(0);
+      if(!(extentReader.getExtent(ulx,uly,lrx,lry))){
+	cerr << "Error: could not get extent from " << mask_opt[0] << endl;
+	exit(1);
+      }
+    }
+    catch(string errorString){
+      maskIsVector=false;
+    }
+  }
+
   ImgWriterOgr activeWriter;
   if(active_opt.size()){
     prob_est_opt[0]=true;
@@ -263,6 +371,28 @@ int main(int argc, char *argv[])
       priors[iclass]/=normPrior;
   }
 
+  //convert start and end band options to vector of band indexes
+  try{
+    if(bstart_opt.size()){
+      if(bend_opt.size()!=bstart_opt.size()){
+	string errorstring="Error: options for start and end band indexes must be provided as pairs, missing end band";
+	throw(errorstring);
+      }
+      band_opt.clear();
+      for(int ipair=0;ipair<bstart_opt.size();++ipair){
+	if(bend_opt[ipair]<=bstart_opt[ipair]){
+	  string errorstring="Error: index for end band must be smaller then start band";
+	  throw(errorstring);
+	}
+	for(int iband=bstart_opt[ipair];iband<=bend_opt[ipair];++iband)
+	  band_opt.push_back(iband);
+      }
+    }
+  }
+  catch(string error){
+    cerr << error << std::endl;
+    exit(1);
+  }
   //sort bands
   if(band_opt.size())
     std::sort(band_opt.begin(),band_opt.end());
@@ -276,7 +406,7 @@ int main(int argc, char *argv[])
   }
 
   //----------------------------------- Training -------------------------------
-  ConfusionMatrix cm;
+  confusionmatrix::ConfusionMatrix cm;
   vector< vector<double> > offset(nbag);
   vector< vector<double> > scale(nbag);
   map<string,Vector2d<float> > trainingMap;
@@ -298,7 +428,7 @@ int main(int argc, char *argv[])
         if(band_opt.size())
           totalSamples=trainingReaderBag.readDataImageOgr(trainingMap,fields,band_opt,label_opt[0],tlayer_opt,verbose_opt[0]);
         else
-          totalSamples=trainingReaderBag.readDataImageOgr(trainingMap,fields,bstart_opt[0],bend_opt[0],label_opt[0],tlayer_opt,verbose_opt[0]);
+          totalSamples=trainingReaderBag.readDataImageOgr(trainingMap,fields,0,0,label_opt[0],tlayer_opt,verbose_opt[0]);
         if(trainingMap.size()<2){
           string errorstring="Error: could not read at least two classes from training file, did you provide class labels in training sample (see option label)?";
           throw(errorstring);
@@ -587,22 +717,24 @@ int main(int argc, char *argv[])
   }//for ibag
   if(cv_opt[0]>1){
     assert(cm.nReference());
+    cm.setFormat(cmformat_opt[0]);
+    cm.reportSE95(false);
     std::cout << cm << std::endl;
-    cout << "class #samples userAcc prodAcc" << endl;
-    double se95_ua=0;
-    double se95_pa=0;
-    double se95_oa=0;
-    double dua=0;
-    double dpa=0;
-    double doa=0;
-    for(short iclass=0;iclass<cm.nClasses();++iclass){
-      dua=cm.ua(cm.getClass(iclass),&se95_ua);
-      dpa=cm.pa(cm.getClass(iclass),&se95_pa);
-      cout << cm.getClass(iclass) << " " << cm.nReference(cm.getClass(iclass)) << " " << dua << " (" << se95_ua << ")" << " " << dpa << " (" << se95_pa << ")" << endl;
-    }
-    std::cout << "Kappa: " << cm.kappa() << std::endl;
-    doa=cm.oa(&se95_oa);
-    std::cout << "Overall Accuracy: " << 100*doa << " (" << 100*se95_oa << ")"  << std::endl;
+    // cout << "class #samples userAcc prodAcc" << endl;
+    // double se95_ua=0;
+    // double se95_pa=0;
+    // double se95_oa=0;
+    // double dua=0;
+    // double dpa=0;
+    // double doa=0;
+    // for(short iclass=0;iclass<cm.nClasses();++iclass){
+    //   dua=cm.ua(cm.getClass(iclass),&se95_ua);
+    //   dpa=cm.pa(cm.getClass(iclass),&se95_pa);
+    //   cout << cm.getClass(iclass) << " " << cm.nReference(cm.getClass(iclass)) << " " << dua << " (" << se95_ua << ")" << " " << dpa << " (" << se95_pa << ")" << endl;
+    // }
+    // std::cout << "Kappa: " << cm.kappa() << std::endl;
+    // doa=cm.oa(&se95_oa);
+    // std::cout << "Overall Accuracy: " << 100*doa << " (" << 100*se95_oa << ")"  << std::endl;
   }
 
   //--------------------------------- end of training -----------------------------------
@@ -636,22 +768,6 @@ int main(int argc, char *argv[])
       cerr << error << std::endl;
       exit(2);
     }
-    ImgReaderGdal maskReader;
-    if(mask_opt.size()){
-      try{
-        if(verbose_opt[0]>=1)
-          std::cout << "opening mask image file " << mask_opt[0] << std::endl;
-        maskReader.open(mask_opt[0]);
-      }
-      catch(string error){
-        cerr << error << std::endl;
-        exit(2);
-      }
-      catch(...){
-        cerr << "error catched" << std::endl;
-        exit(1);
-      }
-    }
     ImgReaderGdal priorReader;
     if(priorimg_opt.size()){
       try{
@@ -722,6 +838,47 @@ int main(int argc, char *argv[])
       cerr << error << std::endl;
     }
   
+    ImgWriterGdal maskWriter;
+
+    if(maskIsVector){
+      try{
+	maskWriter.open("/vsimem/mask.tif",ncol,nrow,1,GDT_Float32,imageType,option_opt);
+	maskWriter.GDALSetNoDataValue(nodata_opt[0]);
+        maskWriter.copyGeoTransform(testImage);
+        maskWriter.setProjection(testImage.getProjection());
+	vector<double> burnValues(1,1);//burn value is 1 (single band)
+	maskWriter.rasterizeOgr(extentReader,burnValues);
+	extentReader.close();
+	maskWriter.close();
+      }
+      catch(string error){
+        cerr << error << std::endl;
+        exit(2);
+      }
+      catch(...){
+        cerr << "error catched" << std::endl;
+        exit(1);
+      }
+      mask_opt.clear();
+      mask_opt.push_back("/vsimem/mask.tif");
+    }
+    ImgReaderGdal maskReader;
+    if(mask_opt.size()){
+      try{
+        if(verbose_opt[0]>=1)
+          std::cout << "opening mask image file " << mask_opt[0] << std::endl;
+        maskReader.open(mask_opt[0]);
+      }
+      catch(string error){
+        cerr << error << std::endl;
+        exit(2);
+      }
+      catch(...){
+        cerr << "error catched" << std::endl;
+        exit(1);
+      }
+    }
+
     for(int iline=0;iline<nrow;++iline){
       vector<float> buffer(ncol);
       vector<short> lineMask;
@@ -747,7 +904,7 @@ int main(int argc, char *argv[])
           }
         }
         else{
-          for(int iband=bstart_opt[0];iband<bstart_opt[0]+nband;++iband){
+          for(int iband=0;iband<nband;++iband){
             if(verbose_opt[0]==2)
               std::cout << "reading band " << iband << std::endl;
             assert(iband>=0);
@@ -791,13 +948,22 @@ int main(int argc, char *argv[])
       //process per pixel
       for(int icol=0;icol<ncol;++icol){
         assert(hpixel[icol].size()==nband);
+	bool doClassify=true;
         bool masked=false;
+	double geox=0;
+	double geoy=0;
+        if(maskIsVector){
+	  doClassify=false;
+	  testImage.image2geo(icol,iline,geox,geoy);
+	  //check enveloppe first
+	  if(uly>=geoy&&lry<=geoy&&ulx<=geox&&lrx>=geox){
+	    doClassify=true;
+	  }
+	}
         if(mask_opt.size()){
 	  //read mask
 	  double colMask=0;
 	  double rowMask=0;
-	  double geox=0;
-	  double geoy=0;
 
 	  testImage.image2geo(icol,iline,geox,geoy);
 	  maskReader.geo2image(geox,geoy,colMask,rowMask);
@@ -826,7 +992,7 @@ int main(int argc, char *argv[])
 		if(lineMask[colMask]==msknodata_opt[ivalue]){
 		  theMask=lineMask[colMask];
 		  masked=true;
-		break;
+		  break;
 		}
 	      }
 	      else{//only values set in msknodata_opt are valid
@@ -855,16 +1021,18 @@ int main(int argc, char *argv[])
 	      break;
 	    }
 	  }
-	  if(!valid){
-	    if(classBag_opt.size())
-	      for(int ibag=0;ibag<nbag;++ibag)
-		classBag[ibag][icol]=nodata_opt[0];
-	    classOut[icol]=nodata_opt[0];
-	    continue;//next column
-	  }
+	  if(!valid)
+	    doClassify=false;
 	}
         for(short iclass=0;iclass<nclass;++iclass)
           probOut[iclass][icol]=0;
+	if(!doClassify){
+	  if(classBag_opt.size())
+	    for(int ibag=0;ibag<nbag;++ibag)
+	      classBag[ibag][icol]=nodata_opt[0];
+	  classOut[icol]=nodata_opt[0];
+	  continue;//next column
+	}
 	if(verbose_opt[0]>1)
 	  std::cout << "begin classification " << std::endl;
         //----------------------------------- classification -------------------
