@@ -21,6 +21,7 @@ along with pktools.  If not, see <http://www.gnu.org/licenses/>.
 #include <assert.h>
 #include <sstream>
 #include <iostream>
+#include <gsl/gsl_cdf.h>
 
 ImgReaderGdal::ImgReaderGdal(void)
   : m_gds(NULL), m_ncol(0), m_nrow(0), m_nband(0)
@@ -424,18 +425,18 @@ double ImgReaderGdal::getMax(int& x, int& y, int band) const{
 
 void ImgReaderGdal::getMinMax(int startCol, int endCol, int startRow, int endRow, int band, double& minValue, double& maxValue) const
 {
-  GDALRasterBand  *poBand;
-  int             bGotMin, bGotMax;
-  double          adfMinMax[2];
+  // GDALRasterBand  *poBand;
+  // int             bGotMin, bGotMax;
+  // double          adfMinMax[2];
         
-  poBand = m_gds->GetRasterBand(band+1);
-  adfMinMax[0] = poBand->GetMinimum( &bGotMin );
-  adfMinMax[1] = poBand->GetMaximum( &bGotMax );
-  if( ! (bGotMin && bGotMax) )
-    GDALComputeRasterMinMax((GDALRasterBandH)poBand, FALSE, adfMinMax);
-    // GDALComputeRasterMinMax((GDALRasterBandH)poBand, TRUE, adfMinMax);
-  minValue=adfMinMax[0];
-  maxValue=adfMinMax[1];
+  // poBand = m_gds->GetRasterBand(band+1);
+  // adfMinMax[0] = poBand->GetMinimum( &bGotMin );
+  // adfMinMax[1] = poBand->GetMaximum( &bGotMax );
+  // if( ! (bGotMin && bGotMax) )
+  //   GDALComputeRasterMinMax((GDALRasterBandH)poBand, FALSE, adfMinMax);
+  //   // GDALComputeRasterMinMax((GDALRasterBandH)poBand, TRUE, adfMinMax);
+  // minValue=adfMinMax[0];
+  // maxValue=adfMinMax[1];
 
   std::vector<double> lineBuffer(endCol-startCol+1);
   bool init=false;
@@ -443,8 +444,6 @@ void ImgReaderGdal::getMinMax(int startCol, int endCol, int startRow, int endRow
   for(int irow=startCol;irow<endRow+1;++irow){
     readData(lineBuffer,GDT_Float64,startCol,endCol,irow,band);
     for(int icol=0;icol<lineBuffer.size();++icol){
-      // bool valid=(find(m_noDataValues.begin(),m_noDataValues.end(),lineBuffer[icol])==m_noDataValues.end());
-      // if(valid){
       if(!isNoData(lineBuffer[icol])){
 	if(!init){
 	  minValue=lineBuffer[icol];
@@ -466,18 +465,6 @@ void ImgReaderGdal::getMinMax(int startCol, int endCol, int startRow, int endRow
 
 void ImgReaderGdal::getMinMax(double& minValue, double& maxValue, int band, bool exhaustiveSearch) const
 {
-  GDALRasterBand  *poBand;
-  int             bGotMin, bGotMax;
-  double          adfMinMax[2];
-        
-  poBand = m_gds->GetRasterBand(band+1);
-  adfMinMax[0] = poBand->GetMinimum( &bGotMin );
-  adfMinMax[1] = poBand->GetMaximum( &bGotMax );
-  if( ! (bGotMin && bGotMax) )
-    GDALComputeRasterMinMax((GDALRasterBandH)poBand, FALSE, adfMinMax);
-    // GDALComputeRasterMinMax((GDALRasterBandH)poBand, TRUE, adfMinMax);
-  minValue=adfMinMax[0];
-  maxValue=adfMinMax[1];
   if(exhaustiveSearch){//force exhaustive search
     std::vector<double> lineBuffer(nrOfCol());
     bool init=false;
@@ -504,18 +491,65 @@ void ImgReaderGdal::getMinMax(double& minValue, double& maxValue, int band, bool
     if(!init)
       throw(static_cast<std::string>("Warning: not initialized"));
   }
+  else{
+    GDALRasterBand  *poBand;
+    int             bGotMin, bGotMax;
+    double          adfMinMax[2];
+        
+    poBand = m_gds->GetRasterBand(band+1);
+    adfMinMax[0] = poBand->GetMinimum( &bGotMin );
+    adfMinMax[1] = poBand->GetMaximum( &bGotMax );
+    if( ! (bGotMin && bGotMax) )
+      GDALComputeRasterMinMax((GDALRasterBandH)poBand, FALSE, adfMinMax);
+    minValue=adfMinMax[0];
+    maxValue=adfMinMax[1];
+    if(m_scale.size()>band){
+      minValue*=m_scale[band];
+      maxValue*=m_scale[band];
+    }
+    if(m_offset.size()>band){
+      minValue+=m_offset[band];
+      maxValue+=m_offset[band];
+    }
+  }
 }
 
-unsigned long int ImgReaderGdal::getHistogram(std::vector<unsigned long int>& histvector, double& min, double& max, unsigned int& nbin, int theBand) const{
+double ImgReaderGdal::getHistogram(std::vector<double>& histvector, double& min, double& max, unsigned int& nbin, int theBand, bool kde){
   double minValue=0;
   double maxValue=0;
-  getMinMax(minValue,maxValue,theBand);
+      
+  if(min>=max)
+    getMinMax(minValue,maxValue,theBand);
+  else{
+    minValue=min;
+    maxValue=max;
+  }
   if(min<max&&min>minValue)
     minValue=min;
   if(min<max&&max<maxValue)
     maxValue=max;
   min=minValue;
   max=maxValue;
+
+  double sigma=0;
+  if(kde){
+    double meanValue=0;
+    double stdDev=0;
+    GDALProgressFunc pfnProgress;
+    void* pProgressData;
+    GDALRasterBand* rasterBand;
+    rasterBand=getRasterBand(theBand);
+    rasterBand->ComputeStatistics(0,&minValue,&maxValue,&meanValue,&stdDev,pfnProgress,pProgressData);
+    //rest minvalue and MaxValue as ComputeStatistics does not account for nodata, scale and offset
+    minValue=min;
+    maxValue=max;
+
+    if(m_scale.size()>theBand){
+      stdDev*=m_scale[theBand];
+    }
+    sigma=1.06*stdDev*pow(getNvalid(theBand),-0.2);
+  }
+
   double scale=0;
   if(maxValue>minValue){
     if(nbin==0)
@@ -525,11 +559,14 @@ unsigned long int ImgReaderGdal::getHistogram(std::vector<unsigned long int>& hi
   else
     nbin=1;
   assert(nbin>0);
-  histvector.resize(nbin);
+  if(histvector.size()!=nbin){
+    histvector.resize(nbin);
+    for(int i=0;i<nbin;histvector[i++]=0);
+  }
+  double nvalid=0;
   unsigned long int nsample=0;
   unsigned long int ninvalid=0;
   std::vector<double> lineBuffer(nrOfCol());
-  for(int i=0;i<nbin;histvector[i++]=0);
   for(int irow=0;irow<nrOfRow();++irow){
     readData(lineBuffer,GDT_Float64,irow,theBand);
     for(int icol=0;icol<nrOfCol();++icol){
@@ -542,10 +579,25 @@ unsigned long int ImgReaderGdal::getHistogram(std::vector<unsigned long int>& hi
       else if(nbin==1)
 	++histvector[0];
       else{//scale to [0:nbin]
-	int theBin=static_cast<unsigned long int>(scale*(lineBuffer[icol]-minValue));
-	assert(theBin>=0);
-	assert(theBin<nbin);
-	++histvector[theBin];
+	if(sigma>0){
+	  //create kde for Gaussian basis function
+	  //todo: speed up by calculating first and last bin with non-zero contriubtion...
+	  //todo: calculate real surface below pdf by using gsl_cdf_gaussian_P(x-mean+binsize,sigma)-gsl_cdf_gaussian_P(x-mean,sigma)
+	  //hiero
+	  for(int ibin=0;ibin<nbin;++ibin){
+	    double icenter=minValue+static_cast<double>(maxValue-minValue)*(ibin+0.5)/nbin;
+	    double thePdf=gsl_ran_gaussian_pdf(lineBuffer[icol]-icenter, sigma);
+	    histvector[ibin]+=thePdf;
+	    nvalid+=thePdf;
+	  }
+	}
+	else{
+	  int theBin=static_cast<unsigned long int>(scale*(lineBuffer[icol]-minValue));
+	  assert(theBin>=0);
+	  assert(theBin<nbin);
+	  ++histvector[theBin];
+	  ++nvalid;
+	}
       // else if(lineBuffer[icol]==maxValue)
       //   ++histvector[nbin-1];
       // else
@@ -553,7 +605,7 @@ unsigned long int ImgReaderGdal::getHistogram(std::vector<unsigned long int>& hi
       }
     }
   }
-  unsigned long int nvalid=nrOfCol()*nrOfRow()-ninvalid;
+  // unsigned long int nvalid=nrOfCol()*nrOfRow()-ninvalid;
   return nvalid;
 }
 
@@ -571,6 +623,26 @@ void ImgReaderGdal::getRange(std::vector<short>& range, int band) const
   sort(range.begin(),range.end());
 }
 
+unsigned long int ImgReaderGdal::getNvalid(int band) const
+{
+  unsigned long int nvalid=0;
+  if(m_noDataValues.size()){
+    std::vector<double> lineBuffer(nrOfCol());
+    for(int irow=0;irow<nrOfRow();++irow){
+      readData(lineBuffer,GDT_Float64,irow,band);
+      for(int icol=0;icol<nrOfCol();++icol){
+	if(isNoData(lineBuffer[icol]))
+	  continue;
+	else
+	  ++nvalid;
+      }
+    }
+    return nvalid;
+  }
+  else
+    return(nrOfCol()*nrOfRow());
+}
+
 int ImgReaderGdal::getNoDataValues(std::vector<double>& noDataValues) const
 {
   if(m_noDataValues.size()){
