@@ -79,8 +79,11 @@ public:
 //   OGRLayer *executeSql(const std::string& output,const std::string& sqlStatement, OGRGeometry* spatialFilter=NULL);
   template<typename T> int readSql(Vector2d<T>& data, const OGRFieldType& fieldType, std::vector<std::string>& fields, const std::string& sqlStatement, OGRGeometry* spatialFilter=NULL, int layer=0, bool pos=false, bool verbose=false);
   template<typename T> int readSql(std::map<int,Vector2d<T> >& data, const OGRFieldType& fieldType, std::vector<std::string>& fields, const std::string& label, const std::string& sqlStatement, OGRGeometry* spatialFilter, int layer=0, bool pos=false, bool verbose=false);
-  bool getExtent(double& ulx, double& uly, double& lrx, double& lry, int layer=0);
+  bool getExtent(double& ulx, double& uly, double& lrx, double& lry, int layer);
+  bool getExtent(double& ulx, double& uly, double& lrx, double& lry);
 
+  void setFieldSeparator(const char fs){ m_fs=fs;};
+  char getFieldSeparator() const { return m_fs;};
   friend std::ostream& operator<<(std::ostream& theOstream, ImgReaderOgr& theImageReader);
   
 protected:
@@ -88,6 +91,7 @@ protected:
 
   std::string m_filename;
   OGRDataSource *m_datasource;
+  char m_fs;
 };
 
 //read data from all features in a map, organized by classes
